@@ -1,16 +1,17 @@
-# Copyright European Organization for Nuclear Research (CERN)
+# -*- coding: utf-8 -*-
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
-# You may not use this file except in compliance with the License.
+# you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# http://www.apache.org/licenses/LICENSE-2.0
 #
-# Authors:
-# - Mario Lassnig, <mario.lassnig@cern.ch>, 2013-2015
-# - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
-# - Eli Chadwick, <eli.chadwick@stfc.ac.uk>, 2020
+#    http://www.apache.org/licenses/LICENSE-2.0
 #
-# PY3K COMPATIBLE
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 """
 Interface for the requests abstraction layer
@@ -22,20 +23,23 @@ from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import api_update_return_dict
 from rucio.core import request
 from rucio.core.rse import get_rse_id
+from rucio.db.sqla.session import read_session, stream_session, transactional_session
 
 
-def queue_requests(requests, issuer, vo='def'):
+@transactional_session
+def queue_requests(requests, issuer, vo='def', session=None):
     """
     Submit transfer or deletion requests on destination RSEs for data identifiers.
 
     :param requests: List of dictionaries containing 'scope', 'name', 'dest_rse_id', 'request_type', 'attributes'
     :param issuer: Issuing account as a string.
     :param vo: The VO to act on.
+    :param session: The database session in use.
     :returns: List of Request-IDs as 32 character hex strings
     """
 
     kwargs = {'requests': requests, 'issuer': issuer}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='queue_requests', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='queue_requests', kwargs=kwargs, session=session):
         raise exception.AccessDenied('%(issuer)s can not queue request' % locals())
 
     for req in requests:
@@ -43,29 +47,12 @@ def queue_requests(requests, issuer, vo='def'):
         if 'account' in req:
             req['account'] = InternalAccount(req['account'], vo=vo)
 
-    new_requests = request.queue_requests(requests)
-    return [api_update_return_dict(r) for r in new_requests]
+    new_requests = request.queue_requests(requests, session=session)
+    return [api_update_return_dict(r, session=session) for r in new_requests]
 
 
-def query_request(request_id, issuer, account, vo='def'):
-    """
-    Query the status of a request.
-
-    :param request_id: Request-ID as a 32 character hex string.
-    :param issuer: Issuing account as a string.
-    :param account: Account identifier as a string.
-    :param vo: The VO to act on.
-    :returns: Request status information as a dictionary.
-    """
-
-    kwargs = {'account': account, 'issuer': issuer, 'request_id': request_id}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='query_request', kwargs=kwargs):
-        raise exception.AccessDenied('%s cannot query request %s' % (account, request_id))
-
-    return request.query_request(request_id)
-
-
-def cancel_request(request_id, issuer, account, vo='def'):
+@transactional_session
+def cancel_request(request_id, issuer, account, vo='def', session=None):
     """
     Cancel a request.
 
@@ -73,16 +60,18 @@ def cancel_request(request_id, issuer, account, vo='def'):
     :param issuer: Issuing account as a string.
     :param account: Account identifier as a string.
     :param vo: The VO to act on.
+    :param session: The database session in use.
     """
 
     kwargs = {'account': account, 'issuer': issuer, 'request_id': request_id}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='cancel_request_', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='cancel_request_', kwargs=kwargs, session=session):
         raise exception.AccessDenied('%s cannot cancel request %s' % (account, request_id))
 
     raise NotImplementedError
 
 
-def cancel_request_did(scope, name, dest_rse, request_type, issuer, account, vo='def'):
+@transactional_session
+def cancel_request_did(scope, name, dest_rse, request_type, issuer, account, vo='def', session=None):
     """
     Cancel a request based on a DID and request type.
 
@@ -93,19 +82,21 @@ def cancel_request_did(scope, name, dest_rse, request_type, issuer, account, vo=
     :param issuer: Issuing account as a string.
     :param account: Account identifier as a string.
     :param vo: The VO to act on.
+    :param session: The database session in use.
     """
 
-    dest_rse_id = get_rse_id(rse=dest_rse, vo=vo)
+    dest_rse_id = get_rse_id(rse=dest_rse, vo=vo, session=session)
 
     kwargs = {'account': account, 'issuer': issuer}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='cancel_request_did', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='cancel_request_did', kwargs=kwargs, session=session):
         raise exception.AccessDenied('%(account)s cannot cancel %(request_type)s request for %(scope)s:%(name)s' % locals())
 
     scope = InternalScope(scope, vo=vo)
-    return request.cancel_request_did(scope, name, dest_rse_id, request_type)
+    return request.cancel_request_did(scope, name, dest_rse_id, request_type, session=session)
 
 
-def get_next(request_type, state, issuer, account, vo='def'):
+@read_session
+def get_next(request_type, state, issuer, account, vo='def', session=None):
     """
     Retrieve the next request matching the request type and state.
 
@@ -114,18 +105,20 @@ def get_next(request_type, state, issuer, account, vo='def'):
     :param issuer: Issuing account as a string.
     :param account: Account identifier as a string.
     :param vo: The VO to act on.
+    :param session: The database session in use.
     :returns: Request as a dictionary.
     """
 
     kwargs = {'account': account, 'issuer': issuer, 'request_type': request_type, 'state': state}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='get_next', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='get_next', kwargs=kwargs, session=session):
         raise exception.AccessDenied('%(account)s cannot get the next request of type %(request_type)s in state %(state)s' % locals())
 
-    reqs = request.get_next(request_type, state)
-    return [api_update_return_dict(r) for r in reqs]
+    reqs = request.get_next(request_type, state, session=session)
+    return [api_update_return_dict(r, session=session) for r in reqs]
 
 
-def get_request_by_did(scope, name, rse, issuer, vo='def'):
+@read_session
+def get_request_by_did(scope, name, rse, issuer, vo='def', session=None):
     """
     Retrieve a request by its DID for a destination RSE.
 
@@ -134,21 +127,48 @@ def get_request_by_did(scope, name, rse, issuer, vo='def'):
     :param rse: The destination RSE of the request as a string.
     :param issuer: Issuing account as a string.
     :param vo: The VO to act on.
+    :param session: The database session in use.
     :returns: Request as a dictionary.
     """
-    rse_id = get_rse_id(rse=rse, vo=vo)
+    rse_id = get_rse_id(rse=rse, vo=vo, session=session)
 
     kwargs = {'scope': scope, 'name': name, 'rse': rse, 'rse_id': rse_id, 'issuer': issuer}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='get_request_by_did', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='get_request_by_did', kwargs=kwargs, session=session):
         raise exception.AccessDenied('%(issuer)s cannot retrieve the request DID %(scope)s:%(name)s to RSE %(rse)s' % locals())
 
     scope = InternalScope(scope, vo=vo)
-    req = request.get_request_by_did(scope, name, rse_id)
+    req = request.get_request_by_did(scope, name, rse_id, session=session)
 
-    return api_update_return_dict(req)
+    return api_update_return_dict(req, session=session)
 
 
-def list_requests(src_rses, dst_rses, states, issuer, vo='def'):
+@read_session
+def get_request_history_by_did(scope, name, rse, issuer, vo='def', session=None):
+    """
+    Retrieve a historical request by its DID for a destination RSE.
+
+    :param scope: The scope of the data identifier as a string.
+    :param name: The name of the data identifier as a string.
+    :param rse: The destination RSE of the request as a string.
+    :param issuer: Issuing account as a string.
+    :param vo: The VO to act on.
+    :param session: The database session in use.
+    :returns: Request as a dictionary.
+    """
+    rse_id = get_rse_id(rse=rse, vo=vo, session=session)
+
+    kwargs = {'scope': scope, 'name': name, 'rse': rse, 'rse_id': rse_id, 'issuer': issuer}
+    if not permission.has_permission(issuer=issuer, vo=vo, action='get_request_history_by_did', kwargs=kwargs, session=session):
+        raise exception.AccessDenied('%(issuer)s cannot retrieve the request DID %(scope)s:%(name)s to RSE %(rse)s' % locals())
+
+    scope = InternalScope(scope, vo=vo)
+    req = request.get_request_history_by_did(scope, name, rse_id, session=session)
+
+    return api_update_return_dict(req, session=session)
+
+
+@stream_session
+def list_requests(src_rses, dst_rses, states, issuer, vo='def', session=None):
     """
     List all requests in a specific state from a source RSE to a destination RSE.
 
@@ -156,14 +176,40 @@ def list_requests(src_rses, dst_rses, states, issuer, vo='def'):
     :param dst_rses: destination RSEs.
     :param states: list of request states.
     :param issuer: Issuing account as a string.
+    :param session: The database session in use.
     """
-    src_rse_ids = [get_rse_id(rse=rse, vo=vo) for rse in src_rses]
-    dst_rse_ids = [get_rse_id(rse=rse, vo=vo) for rse in dst_rses]
+    src_rse_ids = [get_rse_id(rse=rse, vo=vo, session=session) for rse in src_rses]
+    dst_rse_ids = [get_rse_id(rse=rse, vo=vo, session=session) for rse in dst_rses]
 
     kwargs = {'src_rse_id': src_rse_ids, 'dst_rse_id': dst_rse_ids, 'issuer': issuer}
-    if not permission.has_permission(issuer=issuer, vo=vo, action='list_requests', kwargs=kwargs):
+    if not permission.has_permission(issuer=issuer, vo=vo, action='list_requests', kwargs=kwargs, session=session):
         raise exception.AccessDenied('%(issuer)s cannot list requests from RSE %(src_rse)s to RSE %(dst_rse)s' % locals())
 
-    for req in request.list_requests(src_rse_ids, dst_rse_ids, states):
+    for req in request.list_requests(src_rse_ids, dst_rse_ids, states, session=session):
         req = req.to_dict()
-        yield api_update_return_dict(req)
+        yield api_update_return_dict(req, session=session)
+
+
+@stream_session
+def list_requests_history(src_rses, dst_rses, states, issuer, vo='def', offset=None, limit=None, session=None):
+    """
+    List all historical requests in a specific state from a source RSE to a destination RSE.
+
+    :param src_rses: source RSEs.
+    :param dst_rses: destination RSEs.
+    :param states: list of request states.
+    :param issuer: Issuing account as a string.
+    :param offset: offset (for paging).
+    :param limit: limit number of results.
+    :param session: The database session in use.
+    """
+    src_rse_ids = [get_rse_id(rse=rse, vo=vo, session=session) for rse in src_rses]
+    dst_rse_ids = [get_rse_id(rse=rse, vo=vo, session=session) for rse in dst_rses]
+
+    kwargs = {'src_rse_id': src_rse_ids, 'dst_rse_id': dst_rse_ids, 'issuer': issuer}
+    if not permission.has_permission(issuer=issuer, vo=vo, action='list_requests_history', kwargs=kwargs, session=session):
+        raise exception.AccessDenied('%(issuer)s cannot list requests from RSE %(src_rse)s to RSE %(dst_rse)s' % locals())
+
+    for req in request.list_requests_history(src_rse_ids, dst_rse_ids, states, offset, limit, session=session):
+        req = req.to_dict()
+        yield api_update_return_dict(req, session=session)
