@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,28 +12,12 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2018
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2021
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2015
-# - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2018-2021
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019
-# - Ilija Vukotic <ivukotic@cern.ch>, 2020
-# - Luc Goossens <luc.goossens@cern.ch>, 2020
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Eric Vaandering <ewv@fnal.gov>, 2020
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 from datetime import datetime
 from json import dumps, loads
 
 from requests.status_codes import codes
-from six.moves.urllib.parse import quote_plus
+from urllib.parse import quote_plus
 
 from rucio.client.baseclient import BaseClient
 from rucio.client.baseclient import choice
@@ -291,13 +275,12 @@ class ReplicaClient(BaseClient):
         :param files: The list of files. This is a list of DIDs like :
             [{'scope': <scope1>, 'name': <name1>, 'state': <state1>}, {'scope': <scope2>, 'name': <name2>, 'state': <state2>}, ...],
             where a state value can be either of:
-            'A' (available)
-            'S' (suspicious)
-            'U' (unavailable)
-            'R' (recovered)
-            'B' (bad)
-            'L' (lost)
-            'D' (deleted)
+              'A' (AVAILABLE)
+              'U' (UNAVAILABLE)
+              'C' (COPYING)
+              'B' (BEING_DELETED)
+              'D' (BAD)
+              'T' (TEMPORARY_UNAVAILABLE)
         :return: True if replica states have been updated successfully, otherwise an exception is raised.
 
         """
