@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,13 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2018-2020
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2022
 
 import base64
 import datetime
@@ -29,18 +22,17 @@ from hashlib import sha1
 import boto3
 from botocore.client import Config
 from dogpile.cache.api import NO_VALUE
-from oauth2client.service_account import ServiceAccountCredentials
-from six import integer_types
-from six.moves.urllib.parse import urlparse, urlencode
+from google.oauth2.service_account import Credentials
+from urllib.parse import urlparse, urlencode
 
 from rucio.common.cache import make_region_memcached
 from rucio.common.config import config_get, get_rse_credentials
 from rucio.common.exception import UnsupportedOperation
-from rucio.core.monitor import record_timer_block
+from rucio.core.monitor import Timer
 
 CREDS_GCS = None
 
-REGION = make_region_memcached(expiration_time=3600)
+REGION = make_region_memcached(expiration_time=900)
 
 
 def get_signed_url(rse_id, service, operation, url, lifetime=600):
@@ -69,7 +61,7 @@ def get_signed_url(rse_id, service, operation, url, lifetime=600):
         raise UnsupportedOperation('URL must not be empty')
 
     if lifetime:
-        if not isinstance(lifetime, integer_types):
+        if not isinstance(lifetime, int):
             try:
                 lifetime = int(lifetime)
             except:
@@ -78,9 +70,11 @@ def get_signed_url(rse_id, service, operation, url, lifetime=600):
     signed_url = None
     if service == 'gcs':
         if not CREDS_GCS:
-            CREDS_GCS = ServiceAccountCredentials.from_json_keyfile_name(config_get('credentials', 'gcs',
-                                                                                    raise_exception=False,
-                                                                                    default='/opt/rucio/etc/google-cloud-storage-test.json'))
+            CREDS_GCS = Credentials.from_service_account_file(config_get('credentials', 'gcs',
+                                                                         raise_exception=False,
+                                                                         default='/opt/rucio/etc/google-cloud-storage-test.json'))
+        components = urlparse(url)
+        host = components.netloc
 
         # select the correct operation
         operations = {'read': 'GET', 'write': 'PUT', 'delete': 'DELETE'}
@@ -96,20 +90,21 @@ def get_signed_url(rse_id, service, operation, url, lifetime=600):
             lifetime = int(time.mktime(lifetime.timetuple()))
 
         # sign the path only
-        path = urlparse(url).path
+        path = components.path
 
         # assemble message to sign
         to_sign = "%s\n\n\n%s\n%s" % (operation, lifetime, path)
 
         # create URL-capable signature
         # first character is always a '=', remove it
-        signature = urlencode({'': base64.b64encode(CREDS_GCS.sign_blob(to_sign)[1])})[1:]
+        signature = urlencode({'': base64.b64encode(CREDS_GCS.sign_bytes(to_sign))})[1:]
 
         # assemble final signed URL
-        signed_url = 'https://storage.googleapis.com:443%s?GoogleAccessId=%s&Expires=%s&Signature=%s' % (path,
-                                                                                                         CREDS_GCS.service_account_email,
-                                                                                                         lifetime,
-                                                                                                         signature)
+        signed_url = 'https://%s%s?GoogleAccessId=%s&Expires=%s&Signature=%s' % (host,
+                                                                                 path,
+                                                                                 CREDS_GCS.service_account_email,
+                                                                                 lifetime,
+                                                                                 signature)
 
     elif service == 's3':
         # split URL to get hostname, bucket and key
@@ -147,7 +142,7 @@ def get_signed_url(rse_id, service, operation, url, lifetime=600):
         else:
             s3op = 'delete_object'
 
-        with record_timer_block('credential.signs3'):
+        with Timer('credential.signs3'):
             s3 = boto3.client('s3', endpoint_url='https://' + host + ':' + port, aws_access_key_id=access_key, aws_secret_access_key=secret_key, config=Config(signature_version=signature_version, region_name=region_name))
 
             signed_url = s3.generate_presigned_url(s3op, Params={'Bucket': bucket, 'Key': key}, ExpiresIn=lifetime)
@@ -183,7 +178,7 @@ def get_signed_url(rse_id, service, operation, url, lifetime=600):
         expires = int(time.time() + lifetime)
 
         # create signed URL
-        with record_timer_block('credential.signswift'):
+        with Timer('credential.signswift'):
             hmac_body = u'%s\n%s\n%s' % (swiftop, expires, components.path)
             # Python 3 hmac only accepts bytes or bytearray
             sig = hmac.new(bytearray(tempurl_key, 'utf-8'), bytearray(hmac_body, 'utf-8'), sha1).hexdigest()
