@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,14 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2015
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2015-2017
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2019
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2019
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 import datetime
 import hashlib
@@ -88,7 +80,7 @@ def _sanity_check(executable, hostname, hash_executable=None, expiration_delay=D
 
 
 @transactional_session
-def live(executable, hostname, pid, thread, older_than=600, hash_executable=None, payload=None, session=None):
+def live(executable, hostname, pid, thread=None, older_than=600, hash_executable=None, payload=None, session=None):
     """
     Register a heartbeat for a process/thread on a given node.
     The executable name is used for the calculation of thread assignments.
@@ -110,20 +102,27 @@ def live(executable, hostname, pid, thread, older_than=600, hash_executable=None
     if not hash_executable:
         hash_executable = calc_hash(executable)
 
+    if thread:
+        thread_id = thread.ident
+        thread_name = thread.name
+    else:
+        thread_id = 0
+        thread_name = "thread"
+
     # upsert the heartbeat
     rowcount = session.query(Heartbeats)\
         .filter_by(executable=hash_executable,
                    hostname=hostname,
                    pid=pid,
-                   thread_id=thread.ident)\
+                   thread_id=thread_id)\
         .update({'updated_at': datetime.datetime.utcnow(), 'payload': payload})
     if not rowcount:
         Heartbeats(executable=hash_executable,
                    readable=executable,
                    hostname=hostname,
                    pid=pid,
-                   thread_id=thread.ident,
-                   thread_name=thread.name,
+                   thread_id=thread_id,
+                   thread_name=thread_name,
                    payload=payload).save(session=session)
 
     # assign thread identifier
@@ -145,7 +144,7 @@ def live(executable, hostname, pid, thread, older_than=600, hash_executable=None
     # so we have to do it in Python
     assign_thread = 0
     for r in range(len(result)):
-        if result[r][0] == hostname and result[r][1] == pid and result[r][2] == thread.ident:
+        if result[r][0] == hostname and result[r][1] == pid and result[r][2] == thread_id:
             assign_thread = r
             break
 
