@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,45 +12,20 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2015
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2022
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2017
-# - Wen Guan <wen.guan@cern.ch>, 2015-2016
-# - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
-# - Frank Berghaus <frank.berghaus@cern.ch>, 2018
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018-2021
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Brandon White <bjwhite@fnal.gov>, 2019
-# - Gabriele Fronze' <gfronze@cern.ch>, 2019
-# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Tomas Javurek <tomas.javurek@cern.ch>, 2020
-# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
 from datetime import datetime
 import json
+import logging
+import traceback
 from io import StringIO
 from re import match
-from typing import TYPE_CHECKING
+from typing import Any, Dict, Optional, Sequence
 
 import sqlalchemy
 import sqlalchemy.orm
 from dogpile.cache.api import NO_VALUE
-from six import string_types
 from sqlalchemy.exc import DatabaseError, IntegrityError, OperationalError
-from sqlalchemy.orm import aliased
+from sqlalchemy.orm import aliased, Session
 from sqlalchemy.orm.exc import FlushError
 from sqlalchemy.sql.expression import or_, false, func, case
 
@@ -58,22 +33,136 @@ import rucio.core.account_counter
 from rucio.common import exception, utils
 from rucio.common.cache import make_region_memcached
 from rucio.common.config import get_lfn2pfn_algorithm_default
-from rucio.common.utils import CHECKSUM_KEY, is_checksum_valid, GLOBALLY_SUPPORTED_CHECKSUMS
+from rucio.common.utils import CHECKSUM_KEY, GLOBALLY_SUPPORTED_CHECKSUMS, Availability
 from rucio.core.rse_counter import add_counter, get_counter
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import (RSEType, ReplicaState)
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
 
-if TYPE_CHECKING:
-    from typing import Dict, Optional
-    from sqlalchemy.orm import Session
 
 REGION = make_region_memcached(expiration_time=900)
 
 
+class RseData:
+    """
+    Helper data class storing rse data grouped in one place.
+    """
+    def __init__(self, id_, name=None, columns=None, attributes=None, info=None, usage=None, limits=None):
+        self.id = id_
+        self.name = name
+        self.columns = columns
+        self.attributes = attributes
+        self.info = info
+        self.usage = usage
+        self.limits = limits
+
+    def __hash__(self):
+        return hash(self.id)
+
+    def __str__(self):
+        if self.name is not None:
+            return self.name
+        return self.id
+
+    def __eq__(self, other):
+        if other is None:
+            return False
+        return self.id == other.id
+
+    def is_tape(self):
+        if self.info['rse_type'] == RSEType.TAPE or self.info['rse_type'] == 'TAPE':
+            return True
+        return False
+
+    def is_tape_or_staging_required(self):
+        if self.is_tape() or self.attributes.get('staging_required', False):
+            return True
+        return False
+
+    @read_session
+    def ensure_loaded(self, load_name=False, load_columns=False, load_attributes=False,
+                      load_info=False, load_usage=False, load_limits=False, session=None):
+        if self.name is None and load_name:
+            self.name = get_rse_name(rse_id=self.id, session=session)
+        if self.columns is None and load_columns:
+            self.columns = get_rse(rse_id=self.id, session=session)
+        if self.attributes is None and load_attributes:
+            self.attributes = get_rse_attributes(self.id, session=session)
+        if self.info is None and load_info:
+            self.info = get_rse_info(self.id, session=session)
+        if self.usage is None and load_usage:
+            self.usage = get_rse_usage(rse_id=self.id, session=session)
+        if self.limits is None and load_limits:
+            self.limits = get_rse_limits(rse_id=self.id, session=session)
+
+    @staticmethod
+    @read_session
+    def bulk_load(rse_datas: "Sequence[RseData]", load_name=False, load_columns=False, load_attributes=False,
+                  load_info=False, load_usage=False, load_limits=False, session=None):
+        """
+        Given a sequence of RseData objects, ensure that the desired fields are initialised
+        in all objects from the input.
+        """
+        rse_datas_by_id = {}
+        names_to_load = set()
+        columns_to_load = set()
+        attributes_to_load = set()
+        infos_to_load = set()
+        usages_to_load = set()
+        limits_to_load = set()
+        for rse_data in rse_datas:
+            rse_id = rse_data.id
+            rse_datas_by_id.setdefault(rse_id, []).append(rse_data)
+            if load_name and rse_data.name is None:
+                names_to_load.add(rse_id)
+            if load_columns and rse_data.columns is None:
+                columns_to_load.add(rse_id)
+            if load_attributes and rse_data.attributes is None:
+                attributes_to_load.add(rse_id)
+            if load_info and rse_data.info is None:
+                infos_to_load.add(rse_id)
+            if load_usage and rse_data.usage is None:
+                usages_to_load.add(rse_id)
+            if load_limits and rse_data.limits is None:
+                limits_to_load.add(rse_id)
+
+        for rse_id in names_to_load:
+            name = get_rse_name(rse_id=rse_id, session=session)
+            for rse_data in rse_datas_by_id[rse_id]:
+                rse_data.name = name
+
+        for rse_id in columns_to_load:
+            rse = get_rse(rse_id=rse_id, session=session)
+            for rse_data in rse_datas_by_id[rse_id]:
+                rse_data.columns = rse
+
+        for rse_id in attributes_to_load:
+            attributes = get_rse_attributes(rse_id=rse_id, session=session)
+            for rse_data in rse_datas_by_id[rse_id]:
+                rse_data.attributes = attributes
+
+        for rse_id in infos_to_load:
+            info = get_rse_info(rse_id=rse_id, session=session)
+            for rse_data in rse_datas_by_id[rse_id]:
+                rse_data.info = info
+
+        for rse_id in usages_to_load:
+            usage = get_rse_usage(rse_id=rse_id, session=session)
+            for rse_data in rse_datas_by_id[rse_id]:
+                rse_data.usage = usage
+
+        for rse_id in limits_to_load:
+            limits = get_rse_limits(rse_id=rse_id, session=session)
+            for rse_data in rse_datas_by_id[rse_id]:
+                rse_data.limits = limits
+
+
+RSE_SETTINGS = ["continent", "city", "region_code", "country_name", "time_zone", "ISP", "ASN"]
+
+
 @transactional_session
 def add_rse(rse, vo='def', deterministic=True, volatile=False, city=None, region_code=None, country_name=None, continent=None, time_zone=None,
-            ISP=None, staging_area=False, rse_type=RSEType.DISK, longitude=None, latitude=None, ASN=None, availability=7, session=None):
+            ISP=None, staging_area=False, rse_type=RSEType.DISK, longitude=None, latitude=None, ASN=None, availability: Optional[int] = 7, session=None):
     """
     Add a rse with the given location name.
 
@@ -81,27 +170,36 @@ def add_rse(rse, vo='def', deterministic=True, volatile=False, city=None, region
     :param vo: the vo to add the RSE to.
     :param deterministic: Boolean to know if the pfn is generated deterministically.
     :param volatile: Boolean for RSE cache.
-    :param city: City for the RSE.
-    :param region_code: The region code for the RSE.
-    :param country_name: The country.
-    :param continent: The continent.
-    :param time_zone: Timezone.
-    :param ISP: Internet service provider.
+    :param city: City for the RSE. Accessed by `locals()`.
+    :param region_code: The region code for the RSE. Accessed by `locals()`.
+    :param country_name: The country. Accessed by `locals()`.
+    :param continent: The continent. Accessed by `locals()`.
+    :param time_zone: Timezone. Accessed by `locals()`.
+    :param ISP: Internet service provider. Accessed by `locals()`.
     :param staging_area: Staging area.
     :param rse_type: RSE type.
     :param latitude: Latitude coordinate of RSE.
     :param longitude: Longitude coordinate of RSE.
-    :param ASN: Access service network.
+    :param ASN: Access service network. Accessed by `locals()`.
     :param availability: Availability.
+    :param availability_read: If the RSE is readable.
+    :param availability_write: If the RSE is writable.
+    :param availability_delete: If the RSE is deletable.
     :param session: The database session in use.
     """
-    if isinstance(rse_type, string_types):
+    if isinstance(rse_type, str):
         rse_type = RSEType(rse_type)
 
-    new_rse = models.RSE(rse=rse, vo=vo, deterministic=deterministic, volatile=volatile, city=city,
-                         region_code=region_code, country_name=country_name,
-                         continent=continent, time_zone=time_zone, staging_area=staging_area, ISP=ISP, availability=availability,
-                         rse_type=rse_type, longitude=longitude, latitude=latitude, ASN=ASN)
+    availability_values = Availability.from_integer(availability)
+    new_rse = models.RSE(rse=rse, vo=vo, deterministic=deterministic, volatile=volatile,
+                         staging_area=staging_area, rse_type=rse_type, longitude=longitude,
+                         latitude=latitude, availability=availability, availability_read=availability_values.read,
+                         availability_write=availability_values.write, availability_delete=availability_values.delete,
+
+                         # The following fields will be deprecated, they are RSE attributes now.
+                         # (Still in the code for backwards compatibility)
+                         city=city, region_code=region_code, country_name=country_name,
+                         continent=continent, time_zone=time_zone, ISP=ISP, ASN=ASN)
     try:
         new_rse.save(session=session)
     except IntegrityError:
@@ -112,6 +210,13 @@ def add_rse(rse, vo='def', deterministic=True, volatile=False, city=None, region
     # Add rse name as a RSE-Tag
     add_rse_attribute(rse_id=new_rse.id, key=rse, value=True, session=session)
 
+    for setting in RSE_SETTINGS:
+        # The value accessed by locals is defined in the code and it can not be
+        # changed by a user request. This thus does not provide a scurity risk.
+        setting_value = locals().get(setting, None)
+        if setting_value:
+            add_rse_attribute(rse_id=new_rse.id, key=setting, value=setting_value, session=session)
+
     # Add counter to monitor the space usage
     add_counter(rse_id=new_rse.id, session=session)
 
@@ -225,6 +330,21 @@ def rse_is_empty(rse_id, session=None):
     return is_empty
 
 
+@read_session
+def get_rse_settings(rse_id: str, session: Optional[Session] = None) -> Dict[str, Any]:
+    """
+    Get all settings for an RSE.
+
+    :param rse_id:  The rse id.
+    :param session: The database session in use.
+    """
+    attributes = session.query(models.RSEAttrAssociation) \
+        .filter(sqlalchemy.and_(models.RSEAttrAssociation.rse_id == rse_id,
+                                models.RSEAttrAssociation.key.in_(RSE_SETTINGS))) \
+        .all() or []
+    return {attribute.key: attribute.value for attribute in attributes}
+
+
 @read_session
 def get_rse(rse_id, session=None):
     """
@@ -243,6 +363,7 @@ def get_rse(rse_id, session=None):
                                    models.RSE.id == rse_id))\
             .one()
         tmp['type'] = tmp.rse_type
+        tmp.update(get_rse_settings(rse_id, session=session))
         return tmp
     except sqlalchemy.orm.exc.NoResultFound:
         raise exception.RSENotFound('RSE with id \'%s\' cannot be found' % rse_id)
@@ -607,12 +728,27 @@ def get_rse_attribute(key, rse_id=None, value=None, use_cache=True, session=None
     return result
 
 
-@read_session
-def get_rse_supported_checksums(rse_id, session=None):
+def get_rse_attributes(rse_id, session=None):
     """
-    Retrieve from the DB and parse the RSE attribute defining the checksum supported by the RSE
+    List rse attributes
+
+    :param rse:     the rse name.
+    :param rse_id:  The RSE id.
+    :param session: The database session in use.
+
+    :returns: A dictionary with RSE attributes for a RSE.
     """
-    return parse_checksum_support_attribute(get_rse_attribute(key=CHECKSUM_KEY, rse_id=rse_id, session=session))
+
+    key = 'rse_attributes_%s' % (rse_id)
+    result = REGION.get(key)
+    if result is NO_VALUE:
+        try:
+            result = None
+            result = list_rse_attributes(rse_id=rse_id, session=session)
+            REGION.set(key, result)
+        except:
+            logging.warning("Failed to get RSE %s attributes, error: %s" % (rse_id, traceback.format_exc()))
+    return result
 
 
 def get_rse_supported_checksums_from_attributes(rse_attributes):
@@ -636,29 +772,12 @@ def parse_checksum_support_attribute(checksum_attribute):
     if not checksum_attribute:
         return GLOBALLY_SUPPORTED_CHECKSUMS
     else:
-        supported_checksum_list = checksum_attribute[0].split(',')
+        supported_checksum_list = [c.strip() for c in checksum_attribute.split(',') if c.strip()]
         if 'none' in supported_checksum_list:
             return []
         return supported_checksum_list
 
 
-@read_session
-def get_rse_is_checksum_supported(checksum_name, rse_id=None, session=None):
-    """
-    Retrieve RSE attribute value.
-
-    :param checksum_name: The desired checksum name for the attribute.
-    :param rse_id: The RSE id.
-    :param session: The database session in use.
-
-    :returns: True if required checksum is supported, False otherwise.
-    """
-    if is_checksum_valid(checksum_name):
-        return checksum_name in get_rse_supported_checksums(rse_id=rse_id, session=session)
-    else:
-        return False
-
-
 @transactional_session
 def set_rse_usage(rse_id, source, used, free, files=None, session=None):
     """
@@ -703,11 +822,9 @@ def get_rse_usage(rse_id, source=None, session=None, per_account=False):
     if source:
         query_rse_usage = query_rse_usage.filter_by(source=source)
 
-    rse = get_rse_name(rse_id=rse_id, session=session)
     for row in query_rse_usage:
         total = (row.free or 0) + (row.used or 0)
         rse_usage = {'rse_id': rse_id,
-                     'rse': rse,
                      'source': row.source,
                      'used': row.used, 'free': row.free,
                      'total': total,
@@ -926,13 +1043,6 @@ def add_protocol(rse_id, parameter, session=None):
                 parameter[op_name] = parameter['domains'][s][op]
         del parameter['domains']
 
-    # If third_party_copy is set, but not the third_party_copy_read/write,
-    # set the _read/_write by copying the value from the third_party_copy
-    # This is done to maintain compatibility with old scripts.
-    if parameter.get('third_party_copy') is not None and parameter.get('third_party_copy_read') is None and parameter.get('third_party_copy_write') is None:
-        parameter['third_party_copy_read'] = parameter['third_party_copy']
-        parameter['third_party_copy_write'] = parameter['third_party_copy']
-
     if ('extended_attributes' in parameter) and parameter['extended_attributes']:
         try:
             parameter['extended_attributes'] = json.dumps(parameter['extended_attributes'], separators=(',', ':'))
@@ -1041,7 +1151,6 @@ def get_rse_protocols(rse_id, schemes=None, session=None):
                           models.RSEProtocols.read_wan,
                           models.RSEProtocols.write_wan,
                           models.RSEProtocols.delete_wan,
-                          models.RSEProtocols.third_party_copy,
                           models.RSEProtocols.third_party_copy_read,
                           models.RSEProtocols.third_party_copy_write,
                           models.RSEProtocols.extended_attributes).filter(*terms)
@@ -1059,7 +1168,6 @@ def get_rse_protocols(rse_id, schemes=None, session=None):
                  'wan': {'read': row.read_wan,
                          'write': row.write_wan,
                          'delete': row.delete_wan,
-                         'third_party_copy': row.third_party_copy,
                          'third_party_copy_read': row.third_party_copy_read,
                          'third_party_copy_write': row.third_party_copy_write}
              },
@@ -1075,6 +1183,24 @@ def get_rse_protocols(rse_id, schemes=None, session=None):
     return info
 
 
+@read_session
+def get_rse_info(rse_id, session=None):
+    """
+    For historical reasons, related to usage of rsemanager, "rse_info" is equivalent to
+    a cached call to get_rse_protocols without any schemes set.
+
+    :param rse_id: The id of the rse.
+    :param session: The database session.
+    :returns: A dict with RSE information and supported protocols
+    """
+    key = 'rse_info_%s' % rse_id
+    result = REGION.get(key)
+    if result is NO_VALUE:
+        result = get_rse_protocols(rse_id=rse_id, session=session)
+        REGION.set(key, result)
+    return result
+
+
 @transactional_session
 def update_protocols(rse_id, scheme, data, hostname, port, session=None):
     """
@@ -1245,17 +1371,43 @@ def del_protocols(rse_id, scheme, hostname=None, port=None, session=None):
                     i += 1
 
 
+MUTABLE_RSE_PROPERTIES = {
+    'name',
+    'availability_read',
+    'availability_write',
+    'availability_delete',
+    'latitude',
+    'longitude',
+    'time_zone',
+    'rse_type',
+    'volatile',
+    'deterministic',
+    'region_code',
+    'country_name',
+    'city',
+    'staging_area',
+    'qos_class',
+    'continent',
+    'availability'
+}
+
+
 @transactional_session
-def update_rse(rse_id, parameters, session=None):
+def update_rse(rse_id: str, parameters: 'Dict[str, Any]', session=None):
     """
     Update RSE properties like availability or name.
 
     :param rse_id: the id of the new rse.
-    :param  parameters: A dictionnary with property (name, read, write, delete as keys).
+    :param parameters: A dictionary with property (name, read, write, delete as keys).
     :param session: The database session in use.
 
     :raises RSENotFound: If RSE is not found.
+    :raises InputValidationError: If a parameter does not exist. Nothing will be added then.
     """
+    for key in parameters.keys():
+        if key not in MUTABLE_RSE_PROPERTIES:
+            raise exception.InputValidationError(f"The key '{key}' does not exist for RSE properties.")
+
     try:
         query = session.query(models.RSE).filter_by(id=rse_id).one()
     except sqlalchemy.orm.exc.NoResultFound:
@@ -1277,9 +1429,17 @@ def update_rse(rse_id, parameters, session=None):
                 availability = availability | availability_mapping[key]
             else:
                 availability = availability & ~availability_mapping[key]
-        elif key in ['latitude', 'longitude', 'time_zone', 'rse_type', 'volatile', 'deterministic', 'region_code', 'country_name', 'city', 'staging_area', 'qos_class']:
+        elif key in MUTABLE_RSE_PROPERTIES - {'name', 'availability_read', 'availability_write', 'availability_delete'}:
             param[key] = parameters[key]
-    param['availability'] = availability
+
+    if 'availability' not in param:
+        param['availability'] = availability
+
+    if 'availability' in param:
+        availability_values = Availability.from_integer(param['availability'])
+        param['availability_read'] = availability_values.read
+        param['availability_write'] = availability_values.write
+        param['availability_delete'] = availability_values.delete
 
     # handle null-able keys
     for key in parameters:
@@ -1287,6 +1447,12 @@ def update_rse(rse_id, parameters, session=None):
             if param[key] and param[key].lower() in ['', 'none', 'null']:
                 param[key] = None
 
+    # handle rse settings
+    for setting in set(param.keys()).intersection(RSE_SETTINGS):
+        if has_rse_attribute(rse_id, setting, session=session):
+            del_rse_attribute(rse_id, setting, session=session)
+        add_rse_attribute(rse_id, setting, param[setting], session=session)
+
     query.update(param)
     if 'rse' in param:
         add_rse_attribute(rse_id=rse_id, key=parameters['name'], value=True, session=session)
