@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,26 +12,12 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2021
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2015
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2019
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Brandon White <bjwhite@fnal.gov>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2020
-# - Eric Vaandering <ewv@fnal.gov>, 2020
-# - Radu Carpa <radu.carpa@cern.ch>, 2022
 
 import abc
 import re
 
 from dogpile.cache.api import NoValue
 from hashlib import sha256
-from six import add_metaclass
 
 from rucio.common.cache import make_region_memcached
 from rucio.common.exception import InvalidRSEExpression, RSEWriteBlocked
@@ -219,8 +205,7 @@ def __extract_term(expression):
     raise SystemError('This point in the code should not be reachable')
 
 
-@add_metaclass(abc.ABCMeta)
-class BaseExpressionElement:
+class BaseExpressionElement(object, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def resolve_elements(self, session):
         """
@@ -349,8 +334,7 @@ class RSEAttributeLargerCheck(BaseExpressionElement):
         return (set(output), rse_dict)
 
 
-@add_metaclass(abc.ABCMeta)
-class BaseRSEOperator(BaseExpressionElement):
+class BaseRSEOperator(BaseExpressionElement, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def set_left_term(self, left_term):
         """
