@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,18 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2014-2021
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2018
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2015
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - Robert Illingworth <illingwo@fnal.gov>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Luc Goossens <luc.goossens@cern.ch>, 2020
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
-# - Nick Smith <nick.smith@cern.ch>, 2021
 
 import logging
 
@@ -1113,7 +1101,6 @@ def __set_replica_unavailable(replica, session=None):
 
 # # debug helper functions used in apply_rule
 #
-# from __future__ import print_function
 # def prnt(x, header=None):
 #     print()
 #     if header:
@@ -1270,7 +1257,7 @@ def apply_rule(did, rule, rses, source_rses, rseselector, session=None, logger=l
             # simply loop over child datasets
             # this is an approximation because ignoring the possibility of file overlap
             for ds_scope, ds_name in datasets:
-                ds = rucio.core.did.get_did(scope=ds_scope, name=ds_name, dynamic=True, session=session)  # this will be retrieved again later on -> could be optimized
+                ds = rucio.core.did.get_did(scope=ds_scope, name=ds_name, dynamic_depth=DIDType.FILE, session=session)  # this will be retrieved again later on -> could be optimized
                 nbytes += ds['bytes']
                 one_rse_coverage = rucio.core.replica.get_RSEcoverage_of_dataset(scope=ds_scope, name=ds_name, session=session)
                 for rse_id, bytes_ in one_rse_coverage.items():
@@ -1286,7 +1273,7 @@ def apply_rule(did, rule, rses, source_rses, rseselector, session=None, logger=l
         for ds_scope, ds_name in datasets:
             # prnt(('processing dataset ',ds_scope, ds_name))
             #
-            ds = rucio.core.did.get_did(scope=ds_scope, name=ds_name, dynamic=True, session=session)
+            ds = rucio.core.did.get_did(scope=ds_scope, name=ds_name, dynamic_depth=DIDType.FILE, session=session)
             ds_length = ds['length']
             ds_bytes = ds['bytes']
             ds_open = ds['open']
