@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2020 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,22 +12,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Fernando López <felopez@cern.ch>, 2015-2016
-# - Martin Barisits <martin.barisits@cern.ch>, 2017-2019
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2017-2020
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2018
-# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018-2019
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
 
-from __future__ import division
-
-try:
-    import Queue
-except ImportError:
-    import queue as Queue
+import queue as Queue
 import bz2
 import glob
 import logging
