@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,47 +12,26 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2015
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2019
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2013
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2014-2021
-# - Wen Guan <wen.guan@cern.ch>, 2014-2016
-# - Tomáš Kouba <tomas.kouba@cern.ch>, 2014
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2016
-# - dciangot <diego.ciangottini@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Matt Snyder <msnyder@bnl.gov>, 2019-2021
-# - Brandon White <bjwhite@fnal.gov>, 2019
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
-# - Nick Smith <nick.smith@cern.ch>, 2020-2021
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
 
 """
 Conveyor transfer submitter is a daemon to manage non-tape file transfers.
 """
 
-from __future__ import division
-
 import functools
 import logging
 import threading
-import time
 
-from six.moves.configparser import NoOptionError
+from configparser import NoOptionError
 
 import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.config import config_get, config_get_bool, config_get_int
 from rucio.common.logging import setup_logging
 from rucio.common.schema import get_schema_value
-from rucio.core.monitor import MultiCounter, record_timer
-from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, run_conveyor_daemon, next_transfers_to_submit
+from rucio.core.monitor import MultiCounter, record_timer, Timer
+from rucio.core.transfer import transfer_path_str
+from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, next_transfers_to_submit
+from rucio.daemons.common import run_daemon
 from rucio.db.sqla.constants import RequestType
 from rucio.transfertool.fts3 import FTS3Transfertool
 from rucio.transfertool.globus import GlobusTransferTool
@@ -68,18 +47,18 @@ GET_TRANSFERS_COUNTER = MultiCounter(prom='rucio_daemons_conveyor_submitter_get_
                                      documentation='Number of transfers retrieved')
 
 TRANSFERTOOL_CLASSES_BY_NAME = {
-    'fts3': FTS3Transfertool,
-    'globus': GlobusTransferTool,
-    'mock': MockTransfertool,
+    FTS3Transfertool.external_name: FTS3Transfertool,
+    GlobusTransferTool.external_name: GlobusTransferTool,
+    MockTransfertool.external_name: MockTransfertool,
 }
 
 
-def run_once(bulk, group_bulk, filter_transfertool, transfertool, ignore_availability, rse_ids,
+def run_once(bulk, group_bulk, filter_transfertool, transfertools, ignore_availability, rse_ids,
              scheme, failover_scheme, partition_hash_var, timeout, transfertool_kwargs,
              heartbeat_handler, activity):
     worker_number, total_workers, logger = heartbeat_handler.live()
 
-    start_time = time.time()
+    timer = Timer()
     transfers = next_transfers_to_submit(
         total_workers=total_workers,
         worker_number=worker_number,
@@ -90,7 +69,7 @@ def run_once(bulk, group_bulk, filter_transfertool, transfertool, ignore_availab
         rses=rse_ids,
         schemes=scheme,
         filter_transfertool=filter_transfertool,
-        transfertools_by_name={transfertool: TRANSFERTOOL_CLASSES_BY_NAME[transfertool]},
+        transfertool_classes=[TRANSFERTOOL_CLASSES_BY_NAME[transfertool] for transfertool in transfertools],
         older_than=None,
         request_type=RequestType.TRANSFER,
         ignore_availability=ignore_availability,
@@ -98,17 +77,30 @@ def run_once(bulk, group_bulk, filter_transfertool, transfertool, ignore_availab
     )
     total_transfers = len(list(hop for paths in transfers.values() for path in paths for hop in path))
 
-    record_timer('daemons.conveyor.transfer_submitter.get_transfers.per_transfer', (time.time() - start_time) * 1000 / (total_transfers or 1))
+    timer.record('daemons.conveyor.transfer_submitter.get_transfers.per_transfer', divisor=total_transfers or 1)
     GET_TRANSFERS_COUNTER.inc(total_transfers)
     record_timer('daemons.conveyor.transfer_submitter.get_transfers.transfers', total_transfers)
-    logger(logging.INFO, 'Got %s transfers for %s in %s seconds', total_transfers, activity, time.time() - start_time)
+    logger(logging.INFO, 'Got %s transfers for %s in %s seconds', total_transfers, activity, timer.elapsed)
 
     for builder, transfer_paths in transfers.items():
+        # Globus Transfertool is not yet production-ready, but we need to partially activate it
+        # in all submitters if we want to enable native multi-hopping between transfertools.
+        # This "if" can be triggered in a FTS submitter if it tries to multi-hop from
+        # a globus-only RSE via a dual-stack RSE towards an FTS-only RSE.
+        #
+        # Just ignore this transfer and keep it in a queued state, so that it's picked up
+        # latter by that special submitter instance dedicated to globus transfers.
+        #
+        # TODO: remove this "if"
+        if transfertools[0] != GlobusTransferTool.external_name and builder.transfertool_class == GlobusTransferTool:
+            logger(logging.INFO, 'Skipping submission of following transfers: %s', [transfer_path_str(p) for p in transfer_paths])
+            continue
+
         transfertool_obj = builder.make_transfertool(logger=logger, **transfertool_kwargs.get(builder.transfertool_class, {}))
-        start_time = time.time()
+        timer = Timer()
         logger(logging.DEBUG, 'Starting to group transfers for %s (%s)', activity, transfertool_obj)
         grouped_jobs = transfertool_obj.group_into_submit_jobs(transfer_paths)
-        record_timer('daemons.conveyor.transfer_submitter.bulk_group_transfer', (time.time() - start_time) * 1000 / (len(transfer_paths) or 1))
+        timer.record('daemons.conveyor.transfer_submitter.bulk_group_transfer', divisor=len(transfer_paths) or 1)
 
         logger(logging.DEBUG, 'Starting to submit transfers for %s (%s)', activity, transfertool_obj)
         for job in grouped_jobs:
@@ -181,6 +173,7 @@ def submitter(once=False, rses=None, partition_wait_time=10,
     else:
         rse_ids = None
 
+    transfertools = transfertool.split(',')
     transfertool_kwargs = {
         FTS3Transfertool: {
             'group_policy': group_policy,
@@ -197,7 +190,7 @@ def submitter(once=False, rses=None, partition_wait_time=10,
         },
     }
 
-    run_conveyor_daemon(
+    run_daemon(
         once=once,
         graceful_stop=graceful_stop,
         executable=executable,
@@ -209,7 +202,7 @@ def submitter(once=False, rses=None, partition_wait_time=10,
             bulk=bulk,
             group_bulk=group_bulk,
             filter_transfertool=filter_transfertool,
-            transfertool=transfertool,
+            transfertools=transfertools,
             ignore_availability=ignore_availability,
             scheme=scheme,
             failover_scheme=failover_scheme,
