@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,20 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2021
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2021
-# - Wen Guan <wen.guan@cern.ch>, 2014-2015
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2015-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2021
-# - Robert Illingworth <illingwo@fnal.gov>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
-# - Eric Vaandering <ewv@fnal.gov>, 2019-2021
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 '''
    Hermes is a daemon to deliver messages: to a messagebroker via STOMP, or emails via SMTP.
@@ -43,7 +29,6 @@ import traceback
 from email.mime.text import MIMEText
 
 import stomp
-from six import PY2
 from sqlalchemy.orm.exc import NoResultFound
 
 import rucio.db.sqla.util
@@ -107,17 +92,10 @@ def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=60, s
             for message in messages:
                 logger(logging.DEBUG, 'submitting: %s', str(message))
 
-                if PY2:
-                    msg = MIMEText(message['payload']['body'].encode('utf-8'))
-                else:
-                    msg = MIMEText(message['payload']['body'])
-
+                msg = MIMEText(message['payload']['body'])
                 msg['From'] = email_from
                 msg['To'] = ', '.join(message['payload']['to'])
-                if PY2:
-                    msg['Subject'] = message['payload']['subject'].encode('utf-8')
-                else:
-                    msg['Subject'] = message['payload']['subject']
+                msg['Subject'] = message['payload']['subject']
 
                 if send_email:
                     smtp = smtplib.SMTP()
@@ -246,7 +224,8 @@ def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, del
 
             messages = retrieve_messages(bulk=bulk,
                                          thread=heartbeat['assign_thread'],
-                                         total_threads=heartbeat['nr_threads'])
+                                         total_threads=heartbeat['nr_threads'],
+                                         old_mode=True)
 
             if messages:
                 logger(logging.DEBUG, 'retrieved %i messages',
