@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2019-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,13 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 """
 OAuth Manager is a daemon which is reponsible for:
@@ -33,8 +26,6 @@ can be specified by 'max_rows' parameter.
 
 """
 
-from __future__ import print_function
-
 import logging
 import os
 import socket
@@ -47,11 +38,11 @@ from sqlalchemy.exc import DatabaseError
 
 import rucio.db.sqla.util
 from rucio.common.exception import DatabaseException
-from rucio.common.logging import setup_logging
+from rucio.common.logging import setup_logging, formatted_logger
 from rucio.common.utils import daemon_sleep
 from rucio.core.authentication import delete_expired_tokens
 from rucio.core.heartbeat import die, live, sanity_check
-from rucio.core.monitor import record_counter, record_timer
+from rucio.core.monitor import record_counter, Timer
 from rucio.core.oidc import delete_expired_oauthrequests, refresh_jwt_tokens
 
 GRACEFUL_STOP = threading.Event()
@@ -74,48 +65,49 @@ def OAuthManager(once=False, loop_rate=300, max_rows=100, sleep_time=300):
         sleep_time = loop_rate
 
     executable = 'oauth-manager'
-    total_workers = 0
-    worker_number = 0
 
     sanity_check(executable=executable, hostname=socket.gethostname())
 
     # make an initial heartbeat
-    live(executable=executable, hostname=socket.gethostname(), pid=os.getpid(), thread=threading.current_thread())
+    heartbeat = live(executable=executable, hostname=socket.gethostname(), pid=os.getpid(), thread=threading.current_thread())
+    prepend_str = 'oauth_manager [%i/%i] : ' % (heartbeat['assign_thread'], heartbeat['nr_threads'])
+    logger = formatted_logger(logging.log, prepend_str + '%s')
 
     # wait a moment in case all workers started at the same time
     GRACEFUL_STOP.wait(1)
 
     while not GRACEFUL_STOP.is_set():
-        start = time.time()
+        start_time = time.time()
+        timer = Timer()
         # issuing the heartbeat for a second time to make all workers aware of each other
         heartbeat = live(executable=executable, hostname=socket.gethostname(), pid=os.getpid(), thread=threading.current_thread())
-        total_workers = heartbeat['nr_threads']
-        worker_number = heartbeat['assign_thread'] + 1
+        prepend_str = 'oauth_manager [%i/%i] : ' % (heartbeat['assign_thread'], heartbeat['nr_threads'])
+        logger = formatted_logger(logging.log, prepend_str + '%s')
         ndeleted = 0
         ndeletedreq = 0
         nrefreshed = 0
 
         try:
             # ACCESS TOKEN REFRESH - better to run first (in case some of the refreshed tokens needed deletion after this step)
-            logging.info('oauth_manager[%i/%i]: ----- START ----- ACCESS TOKEN REFRESH ----- ', worker_number, total_workers)
-            logging.info('oauth_manager[%i/%i]: starting to query tokens for automatic refresh', worker_number, total_workers)
-            nrefreshed = refresh_jwt_tokens(total_workers, worker_number, refreshrate=int(sleep_time), limit=max_rows)
-            logging.info('oauth_manager[%i/%i]: successfully refreshed %i tokens', worker_number, total_workers, nrefreshed)
-            logging.info('oauth_manager[%i/%i]: ----- END ----- ACCESS TOKEN REFRESH ----- ', worker_number, total_workers)
+            logger(logging.INFO, '----- START ----- ACCESS TOKEN REFRESH ----- ')
+            logger(logging.INFO, 'starting to query tokens for automatic refresh')
+            nrefreshed = refresh_jwt_tokens(heartbeat['nr_threads'], heartbeat['assign_thread'] + 1, refreshrate=int(sleep_time), limit=max_rows)
+            logger(logging.INFO, 'successfully refreshed %i tokens', nrefreshed)
+            logger(logging.INFO, '----- END ----- ACCESS TOKEN REFRESH ----- ')
             record_counter(name='oauth_manager.tokens.refreshed', delta=nrefreshed)
 
         except (DatabaseException, DatabaseError) as err:
             if match('.*QueuePool.*', str(err.args[0])):
-                logging.warning(traceback.format_exc())
+                logger(logging.WARNING, traceback.format_exc())
                 record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
             elif match('.*ORA-03135.*', str(err.args[0])):
-                logging.warning(traceback.format_exc())
+                logger(logging.WARNING, traceback.format_exc())
                 record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
             else:
-                logging.critical(traceback.format_exc())
+                logger(logging.CRITICAL, traceback.format_exc())
                 record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
         except Exception as err:
-            logging.critical(traceback.format_exc())
+            logger(logging.CRITICAL, traceback.format_exc())
             record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
 
         try:
@@ -123,61 +115,61 @@ def OAuthManager(once=False, loop_rate=300, max_rows=100, sleep_time=300):
             # eligible for deletion after refresh might not get dleeted otherwise
             GRACEFUL_STOP.wait(1)
             # EXPIRED TOKEN DELETION
-            logging.info('oauth_manager[%i/%i]: ----- START ----- DELETION OF EXPIRED TOKENS ----- ', worker_number, total_workers)
-            logging.info('oauth_manager[%i/%i]: starting to delete expired tokens', worker_number, total_workers)
-            ndeleted += delete_expired_tokens(total_workers, worker_number, limit=max_rows)
-            logging.info('oauth_manager[%i/%i]: deleted %i expired tokens', worker_number, total_workers, ndeleted)
-            logging.info('oauth_manager[%i/%i]: ----- END ----- DELETION OF EXPIRED TOKENS ----- ', worker_number, total_workers)
+            logger(logging.INFO, '----- START ----- DELETION OF EXPIRED TOKENS ----- ')
+            logger(logging.INFO, 'starting to delete expired tokens')
+            ndeleted += delete_expired_tokens(heartbeat['nr_threads'], heartbeat['assign_thread'] + 1, limit=max_rows)
+            logger(logging.INFO, 'deleted %i expired tokens', ndeleted)
+            logger(logging.INFO, '----- END ----- DELETION OF EXPIRED TOKENS ----- ')
             record_counter(name='oauth_manager.tokens.deleted', delta=ndeleted)
 
         except (DatabaseException, DatabaseError) as err:
             if match('.*QueuePool.*', str(err.args[0])):
-                logging.warning(traceback.format_exc())
+                logger(logging.WARNING, traceback.format_exc())
                 record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
             elif match('.*ORA-03135.*', str(err.args[0])):
-                logging.warning(traceback.format_exc())
+                logger(logging.WARNING, traceback.format_exc())
                 record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
             else:
-                logging.critical(traceback.format_exc())
+                logger(logging.CRITICAL, traceback.format_exc())
                 record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
         except Exception as err:
-            logging.critical(traceback.format_exc())
+            logger(logging.CRITICAL, traceback.format_exc())
             record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
 
         try:
             # DELETING EXPIRED OAUTH SESSION PARAMETERS
-            logging.info('oauth_manager[%i/%i]: ----- START ----- DELETION OF EXPIRED OAUTH SESSION REQUESTS ----- ', worker_number, total_workers)
-            logging.info('oauth_manager[%i/%i]: starting deletion of expired OAuth session requests', worker_number, total_workers)
-            ndeletedreq += delete_expired_oauthrequests(total_workers, worker_number, limit=max_rows)
-            logging.info('oauth_manager[%i/%i]: expired parameters of %i authentication requests were deleted', worker_number, total_workers, ndeletedreq)
-            logging.info('oauth_manager[%i/%i]: ----- END ----- DELETION OF EXPIRED OAUTH SESSION REQUESTS ----- ', worker_number, total_workers)
+            logger(logging.INFO, '----- START ----- DELETION OF EXPIRED OAUTH SESSION REQUESTS ----- ')
+            logger(logging.INFO, 'starting deletion of expired OAuth session requests')
+            ndeletedreq += delete_expired_oauthrequests(heartbeat['nr_threads'], heartbeat['assign_thread'] + 1, limit=max_rows)
+            logger(logging.INFO, 'expired parameters of %i authentication requests were deleted', ndeletedreq)
+            logger(logging.INFO, '----- END ----- DELETION OF EXPIRED OAUTH SESSION REQUESTS ----- ')
             record_counter(name='oauth_manager.oauthreq.deleted', delta=ndeletedreq)
 
         except (DatabaseException, DatabaseError) as err:
             if match('.*QueuePool.*', str(err.args[0])):
-                logging.warning(traceback.format_exc())
+                logger(logging.WARNING, traceback.format_exc())
                 record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
             elif match('.*ORA-03135.*', str(err.args[0])):
-                logging.warning(traceback.format_exc())
+                logger(logging.WARNING, traceback.format_exc())
                 record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
             else:
-                logging.critical(traceback.format_exc())
+                logger(logging.CRITICAL, traceback.format_exc())
                 record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
         except Exception as err:
-            logging.critical(traceback.format_exc())
+            logger(logging.CRITICAL, traceback.format_exc())
             record_counter('oauth_manager.exceptions.{exception}', labels={'exception': err.__class__.__name__})
 
-        tottime = time.time() - start
-        logging.info('oauth_manager[%i/%i]: took %f seconds to delete %i tokens, %i session parameters and refreshed %i tokens', worker_number, total_workers, tottime, ndeleted, ndeletedreq, nrefreshed)
-        record_timer(stat='oauth_manager.duration', time=1000 * tottime)
+        timer.stop()
+        logger(logging.INFO, 'took %f seconds to delete %i tokens, %i session parameters and refreshed %i tokens', timer.elapsed, ndeleted, ndeletedreq, nrefreshed)
+        timer.record('oauth_manager.duration')
 
         if once:
             break
         else:
-            daemon_sleep(start_time=start, sleep_time=sleep_time, graceful_stop=GRACEFUL_STOP)
+            daemon_sleep(start_time=start_time, sleep_time=sleep_time, graceful_stop=GRACEFUL_STOP)
 
     die(executable=executable, hostname=socket.gethostname(), pid=os.getpid(), thread=threading.current_thread())
-    logging.info('oauth_manager[%i/%i]: graceful stop done', worker_number, total_workers)
+    logger(logging.INFO, 'graceful stop done')
 
 
 def run(once=False, threads=1, loop_rate=300, max_rows=100, sleep_time=300):
