@@ -1,5 +1,6 @@
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# Copyright 2019-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,20 +13,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Martin Barisits <martin.barisits@cern.ch>, 2019
-# - Brandon White <bjwhite@fnal.gov>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
-# - Christoph Ames <christoph.ames@physik.uni-muenchen.de>, 2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2022
 
 """
 Suspicious-Replica-Recoverer is a daemon that deals with suspicious replicas based on if they are found available on other RSEs
@@ -33,8 +20,6 @@ or if they are the last remaining copy, on how many suspicious replicas are on a
 Consequently, automatic replica recovery is triggered via necromancer daemon, which creates a rule for such bad replica(s).
 """
 
-from __future__ import print_function
-
 import logging
 import os
 import socket
@@ -44,6 +29,7 @@ import traceback
 from datetime import datetime, timedelta
 from re import match
 from sys import argv
+from typing import List
 
 import json
 
@@ -59,8 +45,9 @@ from rucio.common.constants import SuspiciousAvailability
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.monitor import record_counter
 from rucio.core.did import get_metadata
-from rucio.core.replica import list_replicas, get_suspicious_files, add_bad_pfns, declare_bad_file_replicas
+from rucio.core.replica import list_replicas, get_suspicious_files, add_bad_pfns, declare_bad_file_replicas, get_suspicious_reason
 from rucio.core.rse_expression_parser import parse_expression
+from rucio.core.rule import add_rule
 
 from rucio.core.vo import list_vos
 from rucio.db.sqla.util import get_db_time
@@ -68,12 +55,14 @@ from rucio.db.sqla.util import get_db_time
 GRACEFUL_STOP = threading.Event()
 
 
-def declare_suspicious_replicas_bad(once=False, younger_than=3, nattempts=10, vos=None, limit_suspicious_files_on_rse=5, sleep_time=300):
+def declare_suspicious_replicas_bad(once: bool = False, younger_than: int = 3, nattempts: int = 10, vos: List[str] = None, limit_suspicious_files_on_rse: int = 5, sleep_time: int = 3600, active_mode: bool = False):
     """
     Main loop to check for available replicas which are labeled as suspicious.
 
     Gets a list of suspicious replicas that are listed as AVAILABLE in 'replicas' table
     and available on other RSE. Finds surls of these replicas and declares them as bad.
+    Replicas that are the last remaining copy of a file have additional checks (checksum
+    comparison, etc.) before being declared bad.
 
     :param once: If True, the loop is run just once, otherwise the daemon continues looping until stopped.
     :param younger_than: The number of days since which bad_replicas table will be searched
@@ -167,12 +156,23 @@ def declare_suspicious_replicas_bad(once=False, younger_than=3, nattempts=10, vo
                               'nattempts': nattempts,
                               'exclude_states': ['B', 'R', 'D', 'L', 'T'],
                               'is_suspicious': True}
+            getfileskwargs_nattempts_1 = {'younger_than': younger_than,
+                                          'nattempts': 1,
+                                          'nattempts_exact': True,
+                                          'exclude_states': ['B', 'R', 'D', 'L', 'T'],
+                                          'is_suspicious': True}
 
             for vo in vos:
                 logger(logging.INFO, 'Start replica recovery for VO: %s', vo)
                 recoverable_replicas = {}
                 if vo not in recoverable_replicas:
                     recoverable_replicas[vo] = {}
+                # Separate replicas that have only been declared suspicious once from the rest,
+                # as they will be handled differently and shouldn't be considered when deciding
+                # if an RSE is problematic (due to a high number of suspicious replicas)
+                replicas_nattempts_1 = {}
+                if vo not in replicas_nattempts_1:
+                    replicas_nattempts_1[vo] = {}
 
                 # rse_list = sorted([rse for rse in parse_expression('enable_suspicious_file_recovery=true', filter={'vo': vo})], key=lambda k: k['rse'])
                 rse_list = sorted([rse for rse in parse_expression('enable_suspicious_file_recovery=true') if rse['vo'] == vo], key=lambda k: k['rse'])
@@ -185,12 +185,17 @@ def declare_suspicious_replicas_bad(once=False, younger_than=3, nattempts=10, vo
                     time_start_rse = time.time()
                     rse_expr = rse['rse']
                     cnt_surl_not_found = 0
+                    cnt_surl_not_found_nattempts_1 = 0
                     if rse_expr not in recoverable_replicas[vo]:
                         recoverable_replicas[vo][rse_expr] = {}
+                    if rse_expr not in replicas_nattempts_1[vo]:
+                        replicas_nattempts_1[vo][rse_expr] = {}
                     # Get a dictionary of the suspicious replicas on the RSE that have available copies on other RSEs
                     suspicious_replicas_avail_elsewhere = get_suspicious_files(rse_expr, available_elsewhere=SuspiciousAvailability["EXIST_COPIES"].value, filter_={'vo': vo}, **getfileskwargs)
                     # Get the suspicious replicas that are the last remaining copies
                     suspicious_replicas_last_copy = get_suspicious_files(rse_expr, available_elsewhere=SuspiciousAvailability["LAST_COPY"].value, filter_={'vo': vo}, **getfileskwargs)
+                    # Get the suspicious replicas that have only been declared once
+                    suspicious_replicas_nattempts_1 = get_suspicious_files(rse_expr, available_elsewhere=SuspiciousAvailability["ALL"].value, filter_={'vo': vo}, **getfileskwargs_nattempts_1)
 
                     logger(logging.DEBUG, 'Suspicious replicas on %s:', rse_expr)
                     logger(logging.DEBUG, 'Replicas with copies on other RSEs (%s):', len(suspicious_replicas_avail_elsewhere))
@@ -199,6 +204,9 @@ def declare_suspicious_replicas_bad(once=False, younger_than=3, nattempts=10, vo
                     logger(logging.DEBUG, 'Replicas that are the last remaining copy (%s):', len(suspicious_replicas_last_copy))
                     for i in suspicious_replicas_last_copy:
                         logger(logging.DEBUG, '%s', i)
+                    logger(logging.DEBUG, 'Replicas that have only been declared once (%s):', len(suspicious_replicas_nattempts_1))
+                    for i in suspicious_replicas_nattempts_1:
+                        logger(logging.DEBUG, '%s', i)
 
                     # RSEs that aren't available shouldn't have suspicious replicas showing up. Skip to next RSE.
                     if (rse['availability'] not in {4, 5, 6, 7}) and ((len(suspicious_replicas_avail_elsewhere) > 0) or (len(suspicious_replicas_last_copy) > 0)):
@@ -236,17 +244,64 @@ def declare_suspicious_replicas_bad(once=False, younger_than=3, nattempts=10, vo
                                     cnt_surl_not_found += 1
                                     logger(logging.WARNING, 'Skipping suspicious replica %s on %s, no surls were found.', rep_name, rse_expr)
 
-                    logger(logging.INFO, 'Suspicious replica query took %s seconds on %s and found %i suspicious replicas. The pfns for %s/%s replicas were found.',
-                           time.time() - time_start_rse, rse_expr, len(suspicious_replicas_avail_elsewhere) + len(suspicious_replicas_last_copy),
-                           len(suspicious_replicas_avail_elsewhere) + len(suspicious_replicas_last_copy) - cnt_surl_not_found, len(suspicious_replicas_avail_elsewhere) + len(suspicious_replicas_last_copy))
+                    if suspicious_replicas_nattempts_1:
+                        for replica in suspicious_replicas_nattempts_1:
+                            if vo == replica['scope'].vo:
+                                scope = replica['scope']
+                                rep_name = replica['name']
+                                rse_id = replica['rse_id']
+                                surl_not_found = True
+                                for rep in list_replicas([{'scope': scope, 'name': rep_name}]):
+                                    for rse_ in rep['rses']:
+                                        if rse_ == rse_id:
+                                            replicas_nattempts_1[vo][rse_expr][rep_name] = {'name': rep_name, 'rse_id': rse_id, 'scope': scope, 'surl': rep['rses'][rse_][0], 'available_elsewhere': True}
+                                            surl_not_found = False
+                                if surl_not_found:
+                                    cnt_surl_not_found_nattempts_1 += 1
+                                    logger(logging.WARNING, 'Skipping suspicious replica %s on %s, no surls were found.', rep_name, rse_expr)
+
+                    logger(logging.INFO, 'Suspicious replica query took %s seconds on %s and found %i suspicious replica(s) with a minimum of nattempts=%i. The pfns for %s/%s replicas were found.',
+                           time.time() - time_start_rse,
+                           rse_expr,
+                           len(suspicious_replicas_avail_elsewhere) + len(suspicious_replicas_last_copy),
+                           nattempts,
+                           len(suspicious_replicas_avail_elsewhere) + len(suspicious_replicas_last_copy) - cnt_surl_not_found,
+                           len(suspicious_replicas_avail_elsewhere) + len(suspicious_replicas_last_copy))
+
+                    logger(logging.INFO, 'A total of %i replicas with exactly nattempts=1 were found. The pfns for %s/%s replicas were found.',
+                           len(suspicious_replicas_nattempts_1),
+                           len(suspicious_replicas_nattempts_1) - cnt_surl_not_found_nattempts_1,
+                           len(suspicious_replicas_nattempts_1))
 
                     if len(suspicious_replicas_avail_elsewhere) + len(suspicious_replicas_last_copy) != 0:
                         logger(logging.DEBUG, 'List of replicas on %s for which the pfns have been found:', rse_expr)
                         for i in recoverable_replicas[vo][rse_expr]:
                             logger(logging.DEBUG, '%s', i)
 
-                # Log file is long and hard to read -> implement some spacing
+                    if len(suspicious_replicas_nattempts_1) != 0:
+                        logger(logging.DEBUG, 'List of replicas on %s with nattempts=1 for which the pfns have been found:', rse_expr)
+                        for i in replicas_nattempts_1[vo][rse_expr]:
+                            logger(logging.DEBUG, '%s', i)
+
                 logger(logging.INFO, 'All RSEs have been checked for suspicious replicas. Total time: %s seconds.', time.time() - start)
+
+                logger(logging.INFO, 'Create rules for replicas with nattempts=1.')
+
+                for rse_key in replicas_nattempts_1[vo]:
+                    dids_nattempts_1 = []
+                    for replica_values in replicas_nattempts_1[vo][rse_key].values():
+                        dids = {'scope': replica_values['scope'], 'name': replica_values['name'], 'rse': rse_key}
+                        dids_nattempts_1.append(dids)
+
+                    if active_mode:
+                        # Create as many rules as necessary for the replicas to be picked up by the daemon on the next run
+                        if len(dids_nattempts_1) > 0:
+                            add_rule(dids=dids_nattempts_1, account=InternalAccount('root', vo=vo), copies=nattempts, rse_expression='type=SCRATCHDISK', grouping=None, weight=None, lifetime=24 * 3600, locked=False, subscription_id=None)
+
+                            logger(logging.INFO, 'Rules have been created for %i replicas on %s.', len(dids_nattempts_1), rse_key)
+                        else:
+                            logger(logging.INFO, 'No replicas on %s with nattempts=1.', rse_key)
+
                 logger(logging.INFO, 'Begin check for problematic RSEs.')
                 time_start_check_probl = time.time()
 
@@ -260,12 +315,13 @@ def declare_suspicious_replicas_bad(once=False, younger_than=3, nattempts=10, vo
                         for replica_value in recoverable_replicas[vo][rse_key].values():
                             surls_list.append(replica_value['surl'])
 
-                        add_bad_pfns(pfns=surls_list, account=InternalAccount('root', vo=vo), state='TEMPORARY_UNAVAILABLE', expires_at=datetime.utcnow() + timedelta(days=3))
+                        if active_mode:
+                            add_bad_pfns(pfns=surls_list, account=InternalAccount('root', vo=vo), state='TEMPORARY_UNAVAILABLE', expires_at=datetime.utcnow() + timedelta(days=3))
 
                         logger(logging.INFO, "%s is problematic (more than %s suspicious replicas). Send a Jira ticket for the RSE (to be implemented).", rse_key, limit_suspicious_files_on_rse)
                         logger(logging.INFO, "The following files on %s have been marked as TEMPORARILY UNAVAILABLE:", rse_key)
-                        for rse_values in recoverable_replicas[vo][rse_key].values():
-                            logger(logging.INFO, 'Scope: %s    Name: %s', rse_values['scope'], rse_values['name'])
+                        for replica_values in recoverable_replicas[vo][rse_key].values():
+                            logger(logging.INFO, 'Temporarily unavailable: RSE: %s    Scope: %s    Name: %s    PFN: %s', rse_key, replica_values['scope'], replica_values['name'], replica_values['surl'])
                         # Remove the RSE from the dictionary as it has been dealt with.
                         del recoverable_replicas[vo][rse_key]
 
@@ -273,6 +329,9 @@ def declare_suspicious_replicas_bad(once=False, younger_than=3, nattempts=10, vo
                 for rse in list_problematic_rses:
                     logger(logging.INFO, "%s", rse)
 
+                auditor = 0
+                checksum = 0
+
                 # Label suspicious replicas as bad if they have oher copies on other RSEs (that aren't also marked as suspicious).
                 # If they are the last remaining copies, deal with them differently.
                 for rse_key in list(recoverable_replicas[vo].keys()):
@@ -285,50 +344,68 @@ def declare_suspicious_replicas_bad(once=False, younger_than=3, nattempts=10, vo
                     # Get the rse_id by going to one of the suspicious replicas from that RSE and reading it from there
                     rse_id = list(recoverable_replicas[vo][rse_key].values())[0]['rse_id']
                     for replica_key in list(recoverable_replicas[vo][rse_key].keys()):
+                        file_metadata = get_metadata(recoverable_replicas[vo][rse_key][replica_key]["scope"], recoverable_replicas[vo][rse_key][replica_key]["name"])
+                        recoverable_replicas[vo][rse_key][replica_key]["datatype"] = str(file_metadata["datatype"])
                         if recoverable_replicas[vo][rse_key][replica_key]['available_elsewhere'] is True:
                             # Replicas with other copies on at least one other RSE can safely be labeled as bad
-                            files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key]['surl'])
+                            files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key])
                             # Remove replica from dictionary
                             del recoverable_replicas[vo][rse_key][replica_key]
                         elif recoverable_replicas[vo][rse_key][replica_key]['available_elsewhere'] is False:
                             if (recoverable_replicas[vo][rse_key][replica_key]['name'].startswith("log.")) or (recoverable_replicas[vo][rse_key][replica_key]['name'].startswith("user")):
                                 # Don't keep log files or user files
-                                files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key]['surl'])
+                                files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key])
                                 del recoverable_replicas[vo][rse_key][replica_key]
                             else:
                                 # Deal with replicas based on their metadata.
-                                file_metadata = get_metadata(recoverable_replicas[vo][rse_key][replica_key]["scope"], recoverable_replicas[vo][rse_key][replica_key]["name"])
                                 if file_metadata["datatype"] is None:  # "None" type has no function "split()"
-                                    files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key]['surl'])
+                                    files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key])
                                     continue
                                 for i in json_data:
                                     if i["datatype"] == file_metadata["datatype"].split("_")[-1]:
                                         action = i["action"]
                                         if action == "ignore":
-                                            files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key]['surl'])
+                                            files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key])
                                         elif action == "declare bad":
-                                            files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key]['surl'])
+                                            suspicious_reason = get_suspicious_reason(recoverable_replicas[vo][rse_key][replica_key]["rse_id"],
+                                                                                      recoverable_replicas[vo][rse_key][replica_key]["scope"],
+                                                                                      recoverable_replicas[vo][rse_key][replica_key]["name"],
+                                                                                      nattempts)
+                                            for reason in suspicious_reason:
+                                                if "auditor" in reason["reason"].lower():
+                                                    auditor += 1
+                                                    files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key])
+                                                    break
+                                                elif "checksum" in reason["reason"].lower():
+                                                    checksum += 1
+                                                    files_to_be_declared_bad.append(recoverable_replicas[vo][rse_key][replica_key])
+                                                    break
                                         else:
                                             logger(logging.WARNING, "RSE: %s, replica name %s, surl %s: Match for the metadata 'datatype' (%s) of replica found in json file, but no match for 'action' (%s)",
                                                    rse_key, replica_key, recoverable_replicas[vo][rse_key][replica_key]['surl'], i["datatype"], i["action"])
                                         break
                                     else:
-                                        # If no policy has be set, default to ignoring the file (no action taken).
-                                        files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key]['surl'])
+                                        # If no policy has been set, default to ignoring the file (no action taken).
+                                        files_to_be_ignored.append(recoverable_replicas[vo][rse_key][replica_key])
 
                     logger(logging.INFO, '(%s) Remaining replicas (pfns) that will be ignored:', rse_key)
                     for i in files_to_be_ignored:
-                        logger(logging.INFO, '%s', i)
+                        logger(logging.INFO, 'Ignore: RSE: %s    Scope: %s    Name: %s    Datatype: %s    PFN: %s', rse_key, i["scope"], i["name"], i["datatype"], i["surl"])
                     logger(logging.INFO, '(%s) Remaining replica (pfns) that will be declared BAD:', rse_key)
                     for i in files_to_be_declared_bad:
-                        logger(logging.INFO, '%s', i)
+                        logger(logging.INFO, 'Declare bad: RSE: %s    Scope: %s    Name: %s    Datatype: %s    PFN: %s', rse_key, i["scope"], i["name"], i["datatype"], i["surl"])
 
                     if files_to_be_declared_bad:
                         logger(logging.INFO, 'Ready to declare %s bad replica(s) on %s (RSE id: %s).', len(files_to_be_declared_bad), rse_key, str(rse_id))
+                        logger(logging.INFO, 'Number of replicas with checksum problems: %i', checksum)
+                        logger(logging.INFO, 'Number of replicas that were declared suspicious by the auditor: %i', auditor)
 
-                        declare_bad_file_replicas(pfns=files_to_be_declared_bad, reason='Suspicious. Automatic recovery.', issuer=InternalAccount('root', vo=vo), session=None)
+                        if active_mode:
+                            declare_bad_file_replicas(pfns=files_to_be_declared_bad, reason='Suspicious. Automatic recovery.', issuer=InternalAccount('root', vo=vo), session=None)
 
                         logger(logging.INFO, 'Finished declaring bad replicas on %s.\n', rse_key)
+                    else:
+                        logger(logging.INFO, 'No files were declared bad on %s.\n', rse_key)
 
                 logger(logging.INFO, 'Finished checking for problematic RSEs and declaring bad replicas. Total time: %s seconds.', time.time() - time_start_check_probl)
 
@@ -356,7 +433,7 @@ def declare_suspicious_replicas_bad(once=False, younger_than=3, nattempts=10, vo
     logger(logging.INFO, 'Graceful stop done.')
 
 
-def run(once=False, younger_than=3, nattempts=10, vos=None, limit_suspicious_files_on_rse=5):
+def run(once=False, younger_than=3, nattempts=10, vos=None, limit_suspicious_files_on_rse=5, sleep_time=3600, active_mode=False):
     """
     Starts up the Suspicious-Replica-Recoverer threads.
     """
@@ -376,7 +453,7 @@ def run(once=False, younger_than=3, nattempts=10, vos=None, limit_suspicious_fil
     sanity_check(executable='rucio-replica-recoverer', hostname=socket.gethostname())
 
     if once:
-        declare_suspicious_replicas_bad(once, younger_than, nattempts, vos, limit_suspicious_files_on_rse)
+        declare_suspicious_replicas_bad(once, younger_than, nattempts, vos, limit_suspicious_files_on_rse, sleep_time, active_mode)
     else:
         logger(logging.INFO, 'Suspicious file replicas recovery starting 1 worker.')
         t = threading.Thread(target=declare_suspicious_replicas_bad,
@@ -384,7 +461,9 @@ def run(once=False, younger_than=3, nattempts=10, vos=None, limit_suspicious_fil
                                      'younger_than': younger_than,
                                      'nattempts': nattempts,
                                      'vos': vos,
-                                     'limit_suspicious_files_on_rse': limit_suspicious_files_on_rse})
+                                     'limit_suspicious_files_on_rse': limit_suspicious_files_on_rse,
+                                     'sleep_time': sleep_time,
+                                     'active_mode': active_mode})
         t.start()
         logger(logging.INFO, 'Waiting for interrupts')
 
