@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2019-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,19 +12,10 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Matt Snyder <msnyder@bnl.gov>, 2019-2021
-# - Martin Barisits <martin.barisits@cern.ch>, 2019
-# - Tomas Javurek <tomas.javurek@cern.ch>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2021
-
-from __future__ import print_function
 
 import logging
 
-from six import string_types
-from six.moves.urllib.parse import urlparse
+from urllib.parse import urlparse
 
 from rucio.common import exception
 from rucio.common.extra import import_extras
@@ -39,7 +30,7 @@ if EXTRA_MODULES['globus_sdk']:
 
 
 class GlobusRSEProtocol(RSEProtocol):
-    """ This class is to support Globus as a Rucio RSE protocol.  Inherits from abstract base class RSEProtocol."""
+    """ Implementing access to RSEs using the Globus service as a Rucio RSE protocol. """
 
     def __init__(self, protocol_attr, rse_settings, logger=logging.log):
         """ Initializes the object with information about the referred RSE.
@@ -99,7 +90,7 @@ class GlobusRSEProtocol(RSEProtocol):
             :raises RSEFileNameNotSupported: if the provided PFN doesn't match with the protocol settings
         """
         ret = dict()
-        pfns = [pfns] if isinstance(pfns, string_types) else pfns
+        pfns = [pfns] if isinstance(pfns, str) else pfns
 
         for pfn in pfns:
             parsed = urlparse(pfn)
@@ -205,7 +196,7 @@ class GlobusRSEProtocol(RSEProtocol):
         """
         if self.globus_endpoint_id:
             try:
-                delete_response = send_delete_task(endpoint_id=self.globus_endpoint_id[0], path=path)
+                delete_response = send_delete_task(endpoint_id=self.globus_endpoint_id[0], path=path, logger=self.logger)
             except TransferAPIError as err:
                 print(err)
         else:
@@ -225,7 +216,7 @@ class GlobusRSEProtocol(RSEProtocol):
         """
         if self.globus_endpoint_id:
             try:
-                bulk_delete_response = send_bulk_delete_task(endpoint_id=self.globus_endpoint_id[0], pfns=pfns)
+                bulk_delete_response = send_bulk_delete_task(endpoint_id=self.globus_endpoint_id[0], pfns=pfns, logger=self.logger)
             except TransferAPIError as err:
                 self.logger(logging.WARNING, str(err))
         else:
