@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,16 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
-# - Mayank Sharma <mayank.sharma@cern.ch>, 2021
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
-# - Rakshita Varadarajan <rakshitajps@gmail.com>, 2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2021
-
-from __future__ import print_function
 
 import traceback
 
@@ -77,6 +67,27 @@ def did_client():
     return DIDClient()
 
 
+@pytest.fixture(scope='module')
+def rse_client():
+    from rucio.client.rseclient import RSEClient
+
+    return RSEClient()
+
+
+@pytest.fixture(scope='module')
+def scope_client():
+    from rucio.client.scopeclient import ScopeClient
+
+    return ScopeClient()
+
+
+@pytest.fixture(scope='module')
+def dirac_client():
+    from rucio.client.diracclient import DiracClient
+
+    return DiracClient()
+
+
 @pytest.fixture
 def rest_client():
     from rucio.tests.common import print_response
@@ -138,6 +149,13 @@ def root_account(vo):
     return InternalAccount('root', vo=vo)
 
 
+@pytest.fixture(scope='module')
+def jdoe_account(vo):
+    from rucio.common.types import InternalAccount
+
+    return InternalAccount('jdoe', vo=vo)
+
+
 @pytest.fixture(scope="module")
 def containerized_rses(rucio_client):
     """
@@ -173,6 +191,18 @@ def rse_factory(vo):
         yield factory
 
 
+@pytest.fixture(scope="class")
+def rse_factory_unittest(request, vo):
+    """
+    unittest classes can get access to rse_factory fixture via this fixture
+    """
+    from rucio.tests.temp_factories import TemporaryRSEFactory
+    with TemporaryRSEFactory(vo=vo) as factory:
+        request.cls.rse_factory = factory
+        yield factory
+        factory.cleanup()
+
+
 @pytest.fixture
 def did_factory(vo, mock_scope):
     from rucio.tests.temp_factories import TemporaryDidFactory
@@ -229,6 +259,77 @@ def __get_fixture_param(request):
     return fixture_param
 
 
+def __create_in_memory_db_table(name, *columns, **kwargs):
+    """
+    Create an in-memory temporary table using the sqlite memory driver.
+    Make sqlalchemy  aware of that table by registering it via a
+    declarative base.
+    """
+    import datetime
+    from sqlalchemy import Column, DateTime, CheckConstraint
+    from sqlalchemy.pool import StaticPool
+    from sqlalchemy.schema import Table
+    from sqlalchemy.orm import registry
+    from rucio.db.sqla.models import ModelBase
+    from rucio.db.sqla.session import get_maker, create_engine
+
+    engine = create_engine('sqlite://', connect_args={'check_same_thread': False}, poolclass=StaticPool)
+
+    # Create a class which inherits from ModelBase. This will allow us to use the rucio-specific methods like .save()
+    DeclarativeObj = type('DeclarativeObj{}'.format(name), (ModelBase,), {})
+    # Create a new declarative base and map the previously created object into the base
+    mapper_registry = registry()
+    InMemoryBase = mapper_registry.generate_base(name='InMemoryBase{}'.format(name))
+    table_args = tuple(columns) + tuple(kwargs.get('table_args', ())) + (
+        Column("created_at", DateTime, default=datetime.datetime.utcnow),
+        Column("updated_at", DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow),
+        CheckConstraint('CREATED_AT IS NOT NULL', name=name.upper() + '_CREATED_NN'),
+        CheckConstraint('UPDATED_AT IS NOT NULL', name=name.upper() + '_UPDATED_NN'),
+    )
+    table = Table(
+        name,
+        InMemoryBase.metadata,
+        *table_args
+    )
+    mapper_registry.map_imperatively(DeclarativeObj, table)
+    # Performa actual creation of the in-memory table
+    InMemoryBase.metadata.create_all(engine)
+
+    # Register the new table with the associated engine into the sqlalchemy sessionmaker
+    # In theory, this code must be protected by rucio.db.scla.session._LOCK, but this code will be executed
+    # during test case initialization, so there is no risk here to have concurrent calls from within the
+    # same process
+    senssionmaker = get_maker()
+    senssionmaker.kw.setdefault('binds', {}).update({DeclarativeObj: engine})
+    return DeclarativeObj
+
+
+@pytest.fixture
+def message_mock():
+    """
+    Fixture which overrides the Message table with a private instance
+    """
+    from unittest import mock
+    from rucio.common.utils import generate_uuid
+    from rucio.db.sqla.models import Column, String, PrimaryKeyConstraint, CheckConstraint, Text, Index, GUID
+
+    InMemoryMessage = __create_in_memory_db_table(
+        'message_' + generate_uuid(),
+        Column('id', GUID(), default=generate_uuid),
+        Column('event_type', String(256)),
+        Column('payload', String(4000)),
+        Column('payload_nolimit', Text),
+        Column('services', String(256)),
+        table_args=(PrimaryKeyConstraint('id', name='MESSAGES_ID_PK'),
+                    CheckConstraint('EVENT_TYPE IS NOT NULL', name='MESSAGES_EVENT_TYPE_NN'),
+                    CheckConstraint('PAYLOAD IS NOT NULL', name='MESSAGES_PAYLOAD_NN'),
+                    Index('MESSAGES_SERVICES_IDX', 'services', 'event_type'))
+    )
+
+    with mock.patch('rucio.core.message.Message', new=InMemoryMessage):
+        yield
+
+
 @pytest.fixture
 def core_config_mock(request):
     """
@@ -243,9 +344,8 @@ def core_config_mock(request):
     """
     from unittest import mock
     from rucio.common.utils import generate_uuid
-    from sqlalchemy.pool import StaticPool
-    from rucio.db.sqla.models import ModelBase, BASE, Column, String, PrimaryKeyConstraint
-    from rucio.db.sqla.session import get_session, get_maker, get_engine, create_engine, declarative_base
+    from rucio.db.sqla.models import Column, String, PrimaryKeyConstraint
+    from rucio.db.sqla.session import get_session
 
     # Get the fixture parameters
     table_content = []
@@ -253,25 +353,13 @@ def core_config_mock(request):
     if params:
         table_content = params.get("table_content", table_content)
 
-    # Create an in-memory dropdown replacement table for the "models.Config" table
-    engine = create_engine('sqlite://', connect_args={'check_same_thread': False}, poolclass=StaticPool)
-    InMemoryBase = declarative_base(bind=engine)
-
-    class InMemoryConfig(InMemoryBase, ModelBase):
-        __tablename__ = 'configs_' + generate_uuid()
-        section = Column(String(128))
-        opt = Column(String(128))
-        value = Column(String(4000))
-        _table_args = (PrimaryKeyConstraint('section', 'opt', name='CONFIGS_PK'), )
-
-    InMemoryBase.metadata.create_all()
-
-    # Register the new table with the associated engine into the sqlalchemy sessionmaker
-    # In theory, this code must be protected by rucio.db.scla.session._LOCK, but this code will be executed
-    # during test case initialization, so there is no risk here to have concurrent calls from within the
-    # same process
-    current_engine = get_engine()
-    get_maker().configure(binds={BASE: current_engine, InMemoryBase: engine})
+    InMemoryConfig = __create_in_memory_db_table(
+        'configs_' + generate_uuid(),
+        Column('section', String(128)),
+        Column('opt', String(128)),
+        Column('value', String(4000)),
+        table_args=(PrimaryKeyConstraint('section', 'opt', name='CONFIGS_PK'),),
+    )
 
     # Fill the table with the requested mock data
     session = get_session()()
@@ -349,7 +437,11 @@ def metrics_mock():
     """
 
     from unittest import mock
-    from prometheus_client import CollectorRegistry
+    from prometheus_client import CollectorRegistry, values
 
-    with mock.patch('rucio.core.monitor.REGISTRY', new=CollectorRegistry()) as registry, mock.patch('rucio.core.monitor.COUNTERS', new={}):
+    with mock.patch('rucio.core.monitor.REGISTRY', new=CollectorRegistry()) as registry, \
+            mock.patch('rucio.core.monitor.COUNTERS', new={}), \
+            mock.patch('rucio.core.monitor.GAUGES', new={}), \
+            mock.patch('rucio.core.monitor.TIMINGS', new={}), \
+            mock.patch('prometheus_client.values.ValueClass', new=values.MutexValue):
         yield registry
