@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2021-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,12 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
-# - Mayank Sharma <imptodefeat@gmail.com>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2021
 
 import os
 import shutil
@@ -38,7 +32,6 @@ from rucio.db.sqla import models
 from rucio.db.sqla.constants import DIDType
 from rucio.db.sqla.session import transactional_session
 from rucio.tests.common import file_generator, rse_name_generator
-from six import PY3
 from sqlalchemy import and_, or_
 
 
@@ -138,7 +131,7 @@ class TemporaryRSEFactory:
                 'scheme': scheme,
                 'hostname': '%s.cern.ch' % rse_id,
                 'port': 0,
-                'prefix': '/test/',
+                'prefix': '/test_%s/' % rse_id,
                 'impl': protocol_impl,
                 'domains': {
                     'wan': {
@@ -147,6 +140,11 @@ class TemporaryRSEFactory:
                         'delete': 1,
                         'third_party_copy_read': 1,
                         'third_party_copy_write': 1,
+                    },
+                    'lan': {
+                        'read': 1,
+                        'write': 1,
+                        'delete': 1,
                     }
                 }
             }
@@ -162,7 +160,7 @@ class TemporaryRSEFactory:
         return self._make_rse(scheme='file', protocol_impl='rucio.rse.protocols.posix.Default', add_rse_kwargs=kwargs)
 
     def make_mock_rse(self, **kwargs):
-        return self._make_rse(scheme='MOCK', protocol_impl='rucio.rse.protocols.mock.Default', add_rse_kwargs=kwargs)
+        return self._make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.mock.Default', add_rse_kwargs=kwargs)
 
     def make_xroot_rse(self, **kwargs):
         return self._make_rse(scheme='root', protocol_impl='rucio.rse.protocols.xrootd.Default', add_rse_kwargs=kwargs)
@@ -377,12 +375,8 @@ class TemporaryFileFactory:
             self.non_basedir_files.append(fp)
 
         if data is not None:
-            if PY3:
-                with open(fp, 'w', encoding='utf-8') as f:
-                    f.write(data)
-            else:
-                with open(fp, 'w') as f:
-                    f.write(data)
+            with open(fp, 'w', encoding='utf-8') as f:
+                f.write(data)
         else:
             execute('dd if=/dev/urandom of={0} count={1} bs=1'.format(fp, size))
 
