@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,21 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2020
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2013
-# - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2017
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2018
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2015-2017
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Martin Barisits <martin.barisits@cern.ch>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
 
 import unittest
 from json import loads
@@ -278,18 +263,18 @@ class TestAccountClient(unittest.TestCase):
     def test_add_account_success(self):
         """ ACCOUNT (CLIENTS): create a new account and get information about account."""
         account = account_name_generator()
-        type, email = 'USER', 'rucio@email.com'
-        ret = self.client.add_account(account, type, email)
+        type_, email = 'USER', 'rucio@email.com'
+        ret = self.client.add_account(account, type_, email)
         assert ret
 
         with pytest.raises(Duplicate):
-            self.client.add_account(account, type, email)
+            self.client.add_account(account, type_, email)
 
         with pytest.raises(InvalidObject):
-            self.client.add_account('BAD_ACCOUNT_NAME', type, email)
+            self.client.add_account('BAD_ACCOUNT_NAME', type_, email)
 
         with pytest.raises(InvalidObject):
-            self.client.add_account('toooooooloooooonaccounnnnnnnntnammmmme', type, email)
+            self.client.add_account('toooooooloooooonaccounnnnnnnntnammmmme', type_, email)
 
         acc_info = self.client.get_account(account)
         assert acc_info['account'] == account
@@ -317,8 +302,8 @@ class TestAccountClient(unittest.TestCase):
     def test_update_account(self):
         """ ACCOUNT (CLIENTS): create a new account and update it."""
         account = account_name_generator()
-        type, email = 'USER', 'rucio@email.com'
-        ret = self.client.add_account(account, type, email)
+        type_, email = 'USER', 'rucio@email.com'
+        ret = self.client.add_account(account, type_, email)
         assert ret
         self.client.update_account(account=account, key='status', value='SUSPENDED')
         status = self.client.get_account(account=account)['status']
