@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,35 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
-# - Angelos Molfetas <Angelos.Molfetas@cern.ch>, 2012
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2021
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2014-2018
-# - Cheng-Hsi Chao <cheng-hsi.chao@cern.ch>, 2014
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2015-2022
-# - Martin Barisits <martin.barisits@cern.ch>, 2015-2022
-# - Frank Berghaus <frank.berghaus@cern.ch>, 2017-2018
-# - Tobias Wegner <twegner@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Tomas Javurek <tomas.javurek@cern.ch>, 2020
-# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
-# - Rahul Chauhan <omrahulchauhan@gmail.com>, 2021
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Paul Millar <paul.millar@desy.de>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
-# - jdierkes <joel.dierkes@cern.ch>, 2021
-# - Rakshita Varadarajan <rakshitajps@gmail.com>, 2021
-# - Rob Barnsley <robbarnsley@users.noreply.github.com>, 2021
-
-from __future__ import print_function
 
 import os
 import random
@@ -1103,7 +1074,7 @@ class TestBinRucio(unittest.TestCase):
         exitcode, out, err = execute(cmd)
         print(out, err)
         cmd = 'rucio-admin rse add-protocol --hostname blocklistreplica --scheme file --prefix /rucio --port 0 --impl rucio.rse.protocols.posix.Default ' \
-              '--domain-json \'{"wan": {"read": 1, "write": 1, "delete": 1, "third_party_copy": 1}}\' %s' % tmp_rse
+              '--domain-json \'{"wan": {"read": 1, "write": 1, "delete": 1, "third_party_copy_read": 1, "third_party_copy_write": 1}}\' %s' % tmp_rse
         print(self.marker + cmd)
         exitcode, out, err = execute(cmd)
         print(out, err)
@@ -2118,14 +2089,6 @@ class TestBinRucio(unittest.TestCase):
         print(out, err)
         assert exitcode == 0
 
-    def test_rucio_list_file_replicas_rse_is_deprecated(self):
-        """CLIENT(USER): Warn about deprecated command line args"""
-        cmd = 'rucio list-file-replicas test:file1 --rse MOCK --missing'
-        print(self.marker + cmd)
-        exitcode, out, err = execute(cmd)
-        print(out, err)
-        assert 'The use of --rse is deprecated in favour of --rses and will be removed in 1.27' in err
-
     def test_rucio_list_file_replicas(self):
         """CLIENT(USER): List missing file replicas """
         self.account_client.set_local_account_limit('root', self.def_rse, -1)
@@ -2212,6 +2175,39 @@ class TestBinRucio(unittest.TestCase):
 
         try:
             remove(filename)
-        except OSError as e:
-            if e.args[0] != 2:
-                raise e
+        except OSError as err:
+            if err.args[0] != 2:
+                raise err
+
+    def test_add_lifetime_exception_large_dids_number(self):
+        """ CLIENT(USER): Check that exceptions with more than 1k DIDs are supported """
+        filename = get_tmp_dir() + 'lifetime_exception_many_dids.txt'
+        with open(filename, 'w') as file_:
+            for _ in range(2000):
+                file_.write('%s:%s\n' % (self.user, generate_uuid()))
+
+        # Try adding an exception
+        cmd = 'rucio add-lifetime-exception --inputfile %s --reason "%s" --expiration %s' % (filename, 'Needed for analysis', '2015-10-30')
+        print(cmd)
+        exitcode, out, err = execute(cmd)
+        print(exitcode, out, err)
+        assert exitcode == 0
+        assert "Nothing to submit" in err
+
+        try:
+            remove(filename)
+        except OSError as err:
+            if err.args[0] != 2:
+                raise err
+
+    def test_admin_rse_update_unsupported_option(self):
+        """ ADMIN CLIENT: Rse update should throw an unsupported option exception on an unsupported exception."""
+        exitcode, out, err = execute("rucio-admin rse update --setting test_with_non_existing_option --value 3 --rse {}".format(self.def_rse))
+        print(out, err)
+        assert exitcode != 0
+        assert "There is an error with one of the input parameters.\nDetails: The key 'test_with_non_existing_option' does not exist for RSE properties.\nThis means that the input you provided did not meet all the requirements." in err
+
+        exitcode, out, err = execute("rucio-admin rse update --setting country_name --value France --rse {}".format(self.def_rse))
+        print(out, err)
+        assert exitcode == 0
+        assert not err
