@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,15 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Wen Guan <wen.guan@cern.ch>, 2015-2016
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2016
-# - Martin Barisits <martin.barisits@cern.ch>, 2019-2022
-# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
-# - Mayank Sharma <imptodefeat@gmail.com>, 2021-2022
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 import threading
 import time
@@ -35,7 +26,9 @@ from rucio.common.types import InternalAccount
 from rucio.common.utils import generate_uuid
 from rucio.common.exception import ReplicaNotFound, RequestNotFound
 from rucio.core import config as core_config
+from rucio.core import did as did_core
 from rucio.core import distance as distance_core
+from rucio.core import message as message_core
 from rucio.core import replica as replica_core
 from rucio.core import request as request_core
 from rucio.core import rse as rse_core
@@ -424,7 +417,7 @@ def test_multisource(vo, did_factory, root_account, replica_client, core_config_
 @skip_rse_tests_with_accounts
 @pytest.mark.dirty(reason="leaves files in XRD containers")
 @pytest.mark.noparallel(reason="uses predefined RSEs; runs submitter and receiver")
-def test_multisource_receiver(vo, did_factory, replica_client, root_account, metrics_mock):
+def test_multisource_receiver(vo, did_factory, replica_client, root_account, metrics_mock, message_mock):
     """
     Run receiver as a background thread to automatically handle fts notifications.
     Ensure that a multi-source job in which the first source fails is correctly handled by receiver.
@@ -447,7 +440,11 @@ def test_multisource_receiver(vo, did_factory, replica_client, root_account, met
         # Add non-existing replica which will fail during multisource transfers on the RSE with lower cost (will be the preferred source)
         replica_client.add_replicas(rse=src_rse2, files=[{'scope': did['scope'].external, 'name': did['name'], 'bytes': 1, 'adler32': 'aaaaaaaa'}])
 
-        rule_core.add_rule(dids=[did], account=root_account, copies=1, rse_expression=dst_rse, grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)
+        # submit using indirectly via a dataset to test this case
+        dataset = did_factory.make_dataset()
+        did_core.attach_dids(dids=[did], account=root_account, **dataset)
+        did_core.set_metadata(did['scope'], did['name'], 'datatype', 'RAW')
+        rule_core.add_rule(dids=[dataset], account=root_account, copies=1, rse_expression=dst_rse, grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)
         submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=2, partition_wait_time=None, transfertype='single', filter_transfertool=None)
 
         # After submission, the source rse is the one which will fail
@@ -470,6 +467,20 @@ def test_multisource_receiver(vo, did_factory, replica_client, root_account, met
         # The source was updated to the good one
         assert request['source_rse'] == src_rse1
         assert request['source_rse_id'] == src_rse1_id
+
+        # Test the content of generated messages
+        msgs = message_core.retrieve_messages()
+        msg_submitted = next(msg for msg in msgs if msg['event_type'] == 'transfer-submitted')
+        assert msg_submitted['payload']['request-id'] == request['id']
+        assert msg_submitted['payload']['datatype'] == 'RAW'
+        assert msg_submitted['payload']['datasetScope'] == dataset['scope'].external
+        assert msg_submitted['payload']['dataset'] == dataset['name']
+        msg_done = next(msg for msg in msgs if msg['event_type'] == 'transfer-done')
+        assert msg_done['payload']['request-id'] == request['id']
+        assert msg_done['payload']['datatype'] == 'RAW'
+        assert msg_done['payload']['datasetScope'] == dataset['scope'].external
+        assert msg_done['payload']['dataset'] == dataset['name']
+        assert msg_done['payload']['transfer_link'].startswith('https://fts:8449/')
     finally:
         receiver_graceful_stop.set()
         receiver_thread.join(timeout=5)
@@ -675,7 +686,6 @@ def test_preparer_throttler_submitter(rse_factory, did_factory, root_account, fi
 @pytest.mark.dirty(reason="leaves files in XRD containers")
 @pytest.mark.noparallel(reason="runs submitter; poller and finisher")
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
-    'rucio.common.rse_attributes.REGION',
     'rucio.core.rse.REGION',
     'rucio.rse.rsemanager.RSE_REGION',  # for RSE info
 ]}], indirect=True)
@@ -845,10 +855,9 @@ class FTSWrapper(FTS3Transfertool):
     def on_receive(job_response):
         pass
 
-    @classmethod
-    def _FTS3Transfertool__file_from_transfer(cls, transfer, job_params):
-        file = super()._FTS3Transfertool__file_from_transfer(transfer, job_params)
-        cls.on_submit(file)
+    def _FTS3Transfertool__file_from_transfer(self, transfer):
+        file = super()._FTS3Transfertool__file_from_transfer(transfer)
+        self.on_submit(file)
         return file
 
     def _FTS3Transfertool__bulk_query_responses(self, jobs_response, requests_by_eid):
@@ -925,7 +934,6 @@ def overwrite_on_tape_topology(rse_factory, did_factory, root_account, vo, file_
     ('transfers', 'use_multihop', True)
 ]}], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
-    'rucio.common.rse_attributes.REGION',
     'rucio.core.rse.REGION',
     'rucio.core.rse_expression_parser.REGION',  # The list of multihop RSEs is retrieved by an expression
     'rucio.core.config.REGION',
@@ -955,7 +963,58 @@ def test_overwrite_on_tape(overwrite_on_tape_topology, core_config_mock, caches_
     ('transfers', 'use_multihop', True)
 ]}], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
-    'rucio.common.rse_attributes.REGION',
+    'rucio.core.rse.REGION',
+    'rucio.core.rse_expression_parser.REGION',  # The list of multihop RSEs is retrieved by an expression
+    'rucio.core.config.REGION',
+    'rucio.rse.rsemanager.RSE_REGION',  # for RSE info
+]}], indirect=True)
+def test_overwrite_hops(overwrite_on_tape_topology, core_config_mock, caches_mock, did_factory, file_factory):
+    """
+    Ensure that we request overwrite of intermediate hops on multi-hop transfers towards TAPE RSEs
+    """
+    rse1_id, rse2_id, rse3_id, did1, did2 = overwrite_on_tape_topology(did1_corrupted=False, did2_corrupted=True)
+    did_factory.upload_client.upload(
+        [
+            {
+                'path': file_factory.file_generator(size=3),
+                'rse': rse_core.get_rse_name(rse2_id),
+                'did_scope': did1['scope'].external,
+                'did_name': did1['name'],
+                'no_register': True,
+            }
+        ]
+    )
+    all_rses = [rse1_id, rse2_id, rse3_id]
+
+    submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=10, partition_wait_time=None, transfertype='single', filter_transfertool=None)
+
+    fts_schema_version = FTS3Transfertool(external_host=TEST_FTS_HOST).version()['schema']['major']
+    if fts_schema_version >= 8:
+        # Newer fts version will honor the overwrite_hop
+        request = __wait_for_request_state(dst_rse_id=rse2_id, state=RequestState.DONE, **did1)
+        assert request['state'] == RequestState.DONE
+        request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.FAILED, **did1)
+        assert request['state'] == RequestState.FAILED
+        assert 'Destination file exists and overwrite is not enabled' in request['err_msg']
+    else:
+        # FTS only recently introduced the overwrite_hops parameter. It will be ignored on old
+        # fts versions and the first hop will fail with the  file exists error
+        # TODO: remove this else after FTS 3.12 release and after updating rucio/fts container with the new release
+        request = __wait_for_request_state(dst_rse_id=rse2_id, state=RequestState.FAILED, **did1)
+        assert request['state'] == RequestState.FAILED
+        assert 'Destination file exists and overwrite is not enabled' in request['err_msg']
+        request = __wait_for_request_state(dst_rse_id=rse3_id, state=RequestState.FAILED, **did1)
+        assert request['state'] == RequestState.FAILED
+        assert 'Unused hop in multi-hop' in request['err_msg']
+
+
+@skip_rse_tests_with_accounts
+@pytest.mark.dirty(reason="leaves files in XRD containers")
+@pytest.mark.noparallel(reason="runs submitter; poller and finisher")
+@pytest.mark.parametrize("core_config_mock", [{"table_content": [
+    ('transfers', 'use_multihop', True)
+]}], indirect=True)
+@pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
     'rucio.core.rse.REGION',
     'rucio.core.rse_expression_parser.REGION',  # The list of multihop RSEs is retrieved by an expression
     'rucio.core.config.REGION',
@@ -998,7 +1057,6 @@ def test_file_exists_handled(overwrite_on_tape_topology, core_config_mock, cache
     ('transfers', 'overwrite_corrupted_files', False)
 ]}], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
-    'rucio.common.rse_attributes.REGION',
     'rucio.core.rse.REGION',
     'rucio.core.rse_expression_parser.REGION',  # The list of multihop RSEs is retrieved by an expression
     'rucio.core.config.REGION',
@@ -1235,3 +1293,56 @@ def test_two_multihops_same_intermediate_rse(rse_factory, did_factory, root_acco
     for rse_id in [rse2_id, rse3_id, rse4_id, rse6_id]:
         with pytest.raises(ReplicaNotFound):
             replica_core.get_replica(rse_id=rse_id, **did)
+
+
+@skip_rse_tests_with_accounts
+@pytest.mark.noparallel(reason="runs submitter; poller and finisher")
+def test_checksum_validation(rse_factory, did_factory, root_account):
+    """
+    Ensure that the correct checksum validation strategy is applied on submission
+    """
+    src_rse, src_rse_id = rse_factory.make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.posix.Default')
+    dst_rse1, dst_rse1_id = rse_factory.make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.posix.Default')
+    dst_rse2, dst_rse2_id = rse_factory.make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.posix.Default')
+    dst_rse3, dst_rse3_id = rse_factory.make_rse(scheme='mock', protocol_impl='rucio.rse.protocols.posix.Default')
+    all_rses = [src_rse_id, dst_rse1_id, dst_rse2_id, dst_rse3_id]
+
+    for rse_id in [dst_rse1_id, dst_rse2_id, dst_rse3_id]:
+        distance_core.add_distance(src_rse_id, rse_id, ranking=10)
+    for rse_id in all_rses:
+        rse_core.add_rse_attribute(rse_id, 'fts', TEST_FTS_HOST)
+
+    rse_core.add_rse_attribute(src_rse_id, 'supported_checksums', 'adler32')
+    rse_core.add_rse_attribute(dst_rse1_id, 'verify_checksum', False)
+    rse_core.add_rse_attribute(dst_rse2_id, 'supported_checksums', 'md5')
+    rse_core.add_rse_attribute(dst_rse3_id, 'supported_checksums', 'md5,adler32')
+
+    did = did_factory.upload_test_file(src_rse)
+    replica = replica_core.get_replica(rse_id=src_rse_id, **did)
+
+    rule_core.add_rule(dids=[did], account=root_account, copies=3, rse_expression=f'{dst_rse1}|{dst_rse2}|{dst_rse3}', grouping='ALL', weight=None, lifetime=None, locked=False, subscription_id=None)
+
+    class _FTSWrapper(FTSWrapper):
+        @staticmethod
+        def on_submit(file):
+            # Set the correct checksum on source and simulate a wrong checksum on destination
+            file['sources'] = [set_query_parameters(s_url, {'checksum': replica['adler32']}) for s_url in file['sources']]
+            file['destinations'] = [set_query_parameters(d_url, {'checksum': 'randomString2'}) for d_url in file['destinations']]
+
+    with patch('rucio.daemons.conveyor.submitter.TRANSFERTOOL_CLASSES_BY_NAME') as tt_mock:
+        tt_mock.__getitem__.return_value = _FTSWrapper
+        submitter(once=True, rses=[{'id': rse_id} for rse_id in all_rses], group_bulk=2, partition_wait_time=None, transfertype='single', filter_transfertool=None)
+
+    # Checksum verification disabled on this rse, so the transfer must use source validation and succeed
+    request = __wait_for_request_state(dst_rse_id=dst_rse1_id, state=RequestState.DONE, **did)
+    assert request['state'] == RequestState.DONE
+
+    # No common supported checksum between the source and destination rse. It will verify the destination rse checksum and fail
+    request = __wait_for_request_state(dst_rse_id=dst_rse2_id, state=RequestState.FAILED, **did)
+    assert request['state'] == RequestState.FAILED
+    assert 'User and destination checksums do not match' in request['err_msg']
+
+    # Common checksum exists between the two. It must use "both" validation strategy and fail
+    request = __wait_for_request_state(dst_rse_id=dst_rse3_id, state=RequestState.FAILED, **did)
+    assert 'Source and destination checksums do not match' in request['err_msg']
+    assert request['state'] == RequestState.FAILED
