@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,14 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2021
-# - Rob Barnsley <rob.barnsley@skao.int>, 2022
 
 from copy import deepcopy
 
@@ -246,7 +238,7 @@ class TestDidMetaMongo(unittest.TestCase):
         )
 
     def tearDown(self):
-        self.mongo_meta.drop_database()
+        pass
 
     @pytest.mark.dirty
     def test_set_get_metadata(self):
@@ -552,7 +544,7 @@ class TestDidMetaClient(unittest.TestCase):
 
 
 @pytest.fixture
-def testdid(vo):
+def testdid(vo, file_config_mock):
     did_name = 'testdid_%s' % generate_uuid()
     mock_scope = InternalScope('mock', vo=vo)
     didtype = 'DATASET'
@@ -563,7 +555,12 @@ def testdid(vo):
     delete_dids(dids=[{'name': did_name, 'scope': mock_scope, 'did_type': didtype, 'purge_replicas': True}], account=account)
 
 
-def test_did_set_metadata_bulk_single(testdid):
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
+def test_did_set_metadata_bulk_single(testdid, file_config_mock):
     """ DID (CORE) : Test setting metadata in bulk with a single key-value pair """
     skip_without_json()
 
@@ -577,7 +574,12 @@ def test_did_set_metadata_bulk_single(testdid):
     assert testkey in meta and meta[testkey] == testmeta[testkey]
 
 
-def test_did_set_metadata_bulk_multi(testdid):
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
+def test_did_set_metadata_bulk_multi(testdid, file_config_mock):
     """ DID (CORE) : Test setting metadata in bulk with multiple key-values """
     skip_without_json()
 
@@ -616,7 +618,12 @@ def test_set_dids_metadata_bulk_multi(did_factory):
             assert testkey in meta and meta[testkey] == testmeta[testkey]
 
 
-def test_did_set_metadata_bulk_multi_client(testdid):
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
+def test_did_set_metadata_bulk_multi_client(testdid, file_config_mock):
     """ DID (CLIENT) : Test setting metadata in bulk with multiple key-values """
     skip_without_json()
 
