@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,17 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2014-2016
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2015
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2020
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
 
 import unittest
 
@@ -40,6 +29,8 @@ from rucio.core.rule import add_rule, get_rule
 from rucio.daemons.abacus.account import account_update
 from rucio.daemons.judge.evaluator import re_evaluator
 from rucio.db.sqla.constants import DIDType
+from rucio.db.sqla.models import UpdatedDID
+from rucio.db.sqla.session import transactional_session
 from rucio.tests.common_server import get_vo
 from rucio.tests.test_rule import create_files, tag_generator
 
@@ -54,6 +45,12 @@ class TestJudgeEvaluator(unittest.TestCase):
         else:
             cls.vo = {}
 
+        @transactional_session
+        def __cleanup_updated_dids(session=None):
+            session.query(UpdatedDID).delete()
+
+        __cleanup_updated_dids()
+
         # Add test RSE
         cls.rse1 = 'MOCK'
         cls.rse3 = 'MOCK3'
@@ -148,7 +145,7 @@ class TestJudgeEvaluator(unittest.TestCase):
         account_update(once=True)
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
 
@@ -173,7 +170,7 @@ class TestJudgeEvaluator(unittest.TestCase):
         account_update(once=True)
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -201,7 +198,7 @@ class TestJudgeEvaluator(unittest.TestCase):
         re_evaluator(once=True, did_limit=1000)
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
@@ -235,21 +232,21 @@ class TestJudgeEvaluator(unittest.TestCase):
         add_did(scope, container, DIDType.CONTAINER, self.jdoe)
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
         attach_dids(scope, container, [{'scope': scope, 'name': dataset}], self.jdoe)
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
         attach_dids(scope, container, [{'scope': scope, 'name': dataset}], self.jdoe)
 
         scope = InternalScope('mock', **self.vo)
-        files = create_files(3, scope, self.rse1_id, bytes=100)
+        files = create_files(3, scope, self.rse1_id, bytes_=100)
         dataset = 'dataset_' + str(uuid())
         add_did(scope, dataset, DIDType.DATASET, self.jdoe)
         attach_dids(scope, dataset, files, self.jdoe)
