@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2017-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,10 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2017-2021
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
 import os
 import json
@@ -23,7 +19,7 @@ import json
 from datetime import datetime, timedelta
 
 import pytest
-from six.moves.configparser import NoSectionError
+from configparser import NoSectionError
 
 from rucio.common.exception import UnsupportedOperation, ConfigNotFound
 from rucio.common.policy import REGION
