@@ -1,4 +1,5 @@
-# Copyright 2020 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,16 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
-# - Matt Snyder <msnyder@bnl.gov>, 2021
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2021
 
 from datetime import datetime, timedelta
 
@@ -35,10 +26,12 @@ from rucio.common.exception import ReplicaNotFound, DataIdentifierNotFound
 from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid
 from rucio.core import did as did_core
+from rucio.core import message as message_core
 from rucio.core import replica as replica_core
 from rucio.core import rse as rse_core
 from rucio.core import rule as rule_core
 from rucio.daemons.reaper.reaper import reaper
+from rucio.daemons.reaper.dark_reaper import reaper as dark_reaper
 from rucio.daemons.reaper.reaper import run as run_reaper
 from rucio.db.sqla.models import ConstituentAssociationHistory
 from rucio.db.sqla.session import read_session
@@ -71,15 +64,20 @@ def __add_test_rse_and_replicas(vo, scope, rse_name, names, file_size, epoch_tom
         dids.append({'scope': scope, 'name': file_name})
         replica_core.add_replica(rse_id=rse_id, scope=scope,
                                  name=file_name, bytes_=file_size,
-                                 tombstone=tombstone,
+                                 tombstone=tombstone, meta={'datatype': 'SOME_DATATYPE'},
                                  account=InternalAccount('root', vo=vo), adler32=None, md5=None)
     return rse_name, rse_id, dids
 
 
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
     'rucio.daemons.reaper.reaper.REGION'
 ]}], indirect=True)
-def test_reaper(vo, caches_mock):
+def test_reaper(vo, caches_mock, file_config_mock, message_mock):
     """ REAPER (DAEMON): Test the reaper daemon."""
     [cache_region] = caches_mock
     scope = InternalScope('data13_hip', vo=vo)
@@ -105,11 +103,24 @@ def test_reaper(vo, caches_mock):
     reaper(once=True, rses=[], include_rses=rse_name, exclude_rses=None)
     assert len(list(replica_core.list_replicas(dids, rse_expression=rse_name))) == 200
 
+    msgs = message_core.retrieve_messages()
+    assert len(msgs) == 50  # one for each deleted file
+    assert all(msg['payload']['datatype'] == 'SOME_DATATYPE' for msg in msgs)
 
+    # run dark reaper just to catch the simplest possible errors in this daemon.
+    # TODO: remove this when/if we implement good testing for dark reaper
+    dark_reaper(once=True, rses=[rse_id])
+
+
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
     'rucio.daemons.reaper.reaper.REGION'
 ]}], indirect=True)
-def test_reaper_bulk_delete(vo, caches_mock):
+def test_reaper_bulk_delete(vo, caches_mock, file_config_mock):
     """ REAPER (DAEMON): Mock test the reaper daemon on async bulk delete request."""
     [cache_region] = caches_mock
     scope = InternalScope('data13_hip', vo=vo)
@@ -136,10 +147,15 @@ def test_reaper_bulk_delete(vo, caches_mock):
     assert len(list(replica_core.list_replicas(dids, rse_expression=rse_name))) == 200
 
 
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
     'rucio.daemons.reaper.reaper.REGION'
 ]}], indirect=True)
-def test_reaper_multi_vo_via_run(vo, second_vo, scope_factory, caches_mock):
+def test_reaper_multi_vo_via_run(vo, second_vo, scope_factory, caches_mock, file_config_mock):
     """ MULTI VO (DAEMON): Test that reaper runs on the specified VO(s) """
     [cache_region] = caches_mock
     new_vo = second_vo
@@ -171,10 +187,15 @@ def test_reaper_multi_vo_via_run(vo, second_vo, scope_factory, caches_mock):
     assert len(list(replica_api.list_replicas([{'scope': scope_name, 'name': n} for n in names], rse_expression=rse_name, vo=new_vo))) == 25
 
 
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
     'rucio.daemons.reaper.reaper.REGION'
 ]}], indirect=True)
-def test_reaper_affect_other_vo_via_run(vo, second_vo, scope_factory, caches_mock):
+def test_reaper_affect_other_vo_via_run(vo, second_vo, scope_factory, caches_mock, file_config_mock):
     """ MULTI VO (DAEMON): Test that reaper runs on the specified VO(s) and does not reap others"""
     [cache_region] = caches_mock
     new_vo = second_vo
@@ -206,10 +227,15 @@ def test_reaper_affect_other_vo_via_run(vo, second_vo, scope_factory, caches_moc
     assert len(list(replica_api.list_replicas([{'scope': scope_name, 'name': n} for n in names], rse_expression=rse_name, vo=new_vo))) == 25
 
 
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
     'rucio.daemons.reaper.reaper.REGION'
 ]}], indirect=True)
-def test_reaper_multi_vo(vo, second_vo, scope_factory, caches_mock):
+def test_reaper_multi_vo(vo, second_vo, scope_factory, caches_mock, file_config_mock):
     """ REAPER (DAEMON): Test the reaper daemon with multiple vo."""
     [cache_region] = caches_mock
     new_vo = second_vo
@@ -236,16 +262,21 @@ def test_reaper_multi_vo(vo, second_vo, scope_factory, caches_mock):
     assert len(list(replica_core.list_replicas(dids=dids2, rse_expression=both_rses))) == 200
 
 
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
 @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
     'rucio.daemons.reaper.reaper.REGION'
 ]}], indirect=True)
-def test_archive_removal_impact_on_constituents(rse_factory, did_factory, mock_scope, root_account, caches_mock):
+def test_archive_removal_impact_on_constituents(rse_factory, did_factory, mock_scope, root_account, caches_mock, file_config_mock):
     [cache_region] = caches_mock
     rse_name, rse_id = rse_factory.make_mock_rse()
     scope = mock_scope
     account = root_account
 
-    # Create an 2 archives and 4 files:
+    # Create 2 archives and 4 files:
     # - One only exists in the first archive
     # - One in both, plus another replica, which is not in an archive
     # - One in both, plus another replica, which is not in an archive; and this replica has expired
@@ -290,7 +321,7 @@ def test_archive_removal_impact_on_constituents(rse_factory, did_factory, mock_s
 
     # Run reaper the first time.
     # the expired non-archive replica of c_with_expired_replica must be removed,
-    # but the did must not be remove and it must still remain in the dataset because
+    # but the did must not be removed, and it must still remain in the dataset because
     # it still has the replica from inside the archive
     assert replica_core.get_replica(rse_id=rse_id, **c_with_expired_replica)
     cache_region.invalidate()
@@ -337,7 +368,7 @@ def test_archive_removal_impact_on_constituents(rse_factory, did_factory, mock_s
 
     # Expire the second archive replica and run reaper another time
     # c_with_expired_replica is removed because its external replica got removed at previous step
-    # and it exist only inside the archive now.
+    # and it exists only inside the archive now.
     # If not open, Dataset2 will be removed because it will be empty.
     did_core.set_status(open=False, **dataset2)
     replica_core.set_tombstone(rse_id=rse_id, tombstone=datetime.utcnow() - timedelta(days=1), **archive2)
@@ -367,6 +398,11 @@ def test_archive_removal_impact_on_constituents(rse_factory, did_factory, mock_s
     assert __get_archive_contents_history_count(archive2) == 3
 
 
+@pytest.mark.parametrize("file_config_mock", [
+    # Run test twice: with, and without, temp tables
+    {"overrides": [('core', 'use_temp_tables', 'True')]},
+    {"overrides": [('core', 'use_temp_tables', 'False')]},
+], indirect=True)
 @pytest.mark.parametrize("core_config_mock", [{"table_content": [
     ('deletion', 'archive_dids', True), ('deletion', 'archive_content', True)
 ]}], indirect=True)
@@ -375,7 +411,7 @@ def test_archive_removal_impact_on_constituents(rse_factory, did_factory, mock_s
     'rucio.core.config.REGION',
     'rucio.core.replica.REGION',
 ]}], indirect=True)
-def test_archive_of_deleted_dids(vo, did_factory, root_account, core_config_mock, caches_mock):
+def test_archive_of_deleted_dids(vo, did_factory, root_account, core_config_mock, caches_mock, file_config_mock):
     """ REAPER (DAEMON): Test that the options to keep the did and content history work."""
     [reaper_cache_region, _config_cache_region, _replica_cache_region] = caches_mock
     scope = InternalScope('data13_hip', vo=vo)
