@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,29 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2018
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2013-2014
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2022
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2021
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2014
-# - Martin Barisits <martin.barisits@cern.ch>, 2015-2021
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
-# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
-# - Tobias Wegner <twegner@cern.ch>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Luc Goossens <luc.goossens@cern.ch>, 2020
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Ilija Vukotic <ivukotic@uchicago.edu>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
-
-from __future__ import print_function
 
 import hashlib
 import time
@@ -57,10 +34,10 @@ from rucio.core.did import add_did, attach_dids, get_did, set_status, list_files
 from rucio.core.replica import (add_replica, add_replicas, delete_replicas, get_replicas_state,
                                 get_replica, list_replicas, update_replica_state,
                                 get_RSEcoverage_of_dataset, get_replica_atime,
-                                touch_replica, get_bad_pfns, set_tombstone)
+                                touch_replica, get_bad_pfns, set_tombstone, add_bad_dids)
 from rucio.core.rse import add_protocol, add_rse_attribute, del_rse_attribute
-from rucio.daemons.badreplicas.minos import run as minos_run
-from rucio.daemons.badreplicas.minos_temporary_expiration import run as minos_temp_run
+from rucio.daemons.badreplicas.minos import minos
+from rucio.daemons.badreplicas.minos_temporary_expiration import minos_tu_expiration
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import DIDType, ReplicaState, BadPFNStatus, OBSOLETE
 from rucio.db.sqla.session import transactional_session
@@ -215,7 +192,12 @@ class TestReplicaCore:
 
         assert nbfiles == replica_cpt
 
-    def test_delete_replicas(self, rse_factory, mock_scope, root_account):
+    @pytest.mark.parametrize("file_config_mock", [
+        # Run test twice: with, and without, temp tables
+        {"overrides": [('core', 'use_temp_tables', 'True')]},
+        {"overrides": [('core', 'use_temp_tables', 'False')]},
+    ], indirect=True)
+    def test_delete_replicas(self, rse_factory, mock_scope, root_account, file_config_mock):
         """ REPLICA (CORE): Delete replicas """
         _, rse1_id = rse_factory.make_mock_rse()
         _, rse2_id = rse_factory.make_mock_rse()
@@ -229,6 +211,9 @@ class TestReplicaCore:
         add_replicas(rse_id=rse1_id, files=files2, account=root_account, ignore_availability=True)
         add_replicas(rse_id=rse2_id, files=files2, account=root_account, ignore_availability=True)
 
+        # calling delete_replicas without any files must not fail
+        delete_replicas(rse_id=rse1_id, files=[])
+
         delete_replicas(rse_id=rse1_id, files=files1 + files2)
 
         for file in files1:
@@ -569,6 +554,26 @@ class TestReplicaCore:
         assert len(list(list_replicas([{'scope': mock_scope, 'name': dsn}], updated_after=t2))) == 1
         assert len(list(list_replicas([{'scope': mock_scope, 'name': dsn}], updated_after=t3))) == 0
 
+    def test_add_bad_dids(self, rse_factory, mock_scope, root_account):
+        """ REPLICA (CORE): Declare a list of replicas as bad.  """
+        _, rse_id = rse_factory.make_mock_rse()
+        dsn = 'ds_ua_test_dsn_%s' % generate_uuid()
+        add_did(scope=mock_scope, name=dsn, did_type='DATASET', account=root_account)
+
+        dids = [{'scope': mock_scope, 'name': f'{dsn}._{file_index}.data'} for file_index in range(3)]
+
+        for did in dids:
+            add_replica(rse_id=rse_id, scope=mock_scope, name=did["name"], bytes_=12345, account=root_account)
+        attach_dids(scope=mock_scope, name=dsn, dids=dids, account=root_account)
+
+        for did in dids:
+            did["scope"] = str(did["scope"])
+
+        add_bad_dids(dids, rse_id, reason="This DID was bad, it needs to be punished.", issuer=root_account)
+
+        for replica in list(list_replicas([{'scope': mock_scope, 'name': dsn}], all_states=True)):
+            assert replica["states"][rse_id] == "BAD"
+
     def test_get_RSE_coverage_of_dataset(self, rse_factory, mock_scope, root_account):
         """ REPLICA (CORE): test RSE coverage retrieval """
         _, rse1_id = rse_factory.make_mock_rse()
@@ -818,7 +823,7 @@ def test_client_add_temporary_unavailable_pfns(rse_factory, mock_scope, replica_
         replica_client.add_bad_pfns(pfns=list_rep, reason=str(reason_str), state='BADSTATE', expires_at=now.isoformat())
 
     # Run minos once
-    minos_run(threads=1, bulk=10000, once=True)
+    minos(bulk=10000, once=True)
     result = get_bad_pfns(limit=10000, thread=None, total_threads=None, session=None)
     pfns = [res['pfn'] for res in result if res['pfn'] in bad_pfns]
     res_pfns = []
@@ -834,14 +839,17 @@ def test_client_add_temporary_unavailable_pfns(rse_factory, mock_scope, replica_
 
     rep = []
     for did in files:
+        print(did)
         did['state'] = ReplicaState.TEMPORARY_UNAVAILABLE
         rep.append(did)
 
+    time.sleep(10)  # Test broken if minos_tu_expiration excuted immediately. Threading effect ?
     # Run the minos expiration
-    minos_temp_run(threads=1, once=True)
+    minos_tu_expiration(once=True)
     # Check the state in the replica table
     for did in files:
         rep = get_replicas_state(scope=mock_scope, name=did['name'])
+        print(rep)
         assert list(rep.keys())[0] == ReplicaState.AVAILABLE
 
 
@@ -974,7 +982,7 @@ class TestReplicaMetalink:
         rse_info = rsemgr.get_rse_info(rse=rse, vo=vo)
         assert rse_info['deterministic'] is False
         files = [{'scope': mock_scope, 'name': 'file_%s' % generate_uuid(), 'bytes': 1, 'adler32': '0cc737eb',
-                  'pfn': 'srm://%s.cern.ch/srm/managerv2?SFN=/test/%s/%s' % (rse_id, mock_scope, generate_uuid()), 'meta': {'events': 10}} for _ in range(nbfiles)]
+                  'pfn': 'srm://%s.cern.ch/srm/managerv2?SFN=/test_%s/%s/%s' % (rse_id, rse_id, mock_scope, generate_uuid()), 'meta': {'events': 10}} for _ in range(nbfiles)]
         for f in files:
             input_[f['pfn']] = {'scope': f['scope'].external, 'name': f['name']}
         add_replicas(rse_id=rse_id, files=files, account=root_account, ignore_availability=True)
