@@ -1,5 +1,6 @@
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# Copyright 2019-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,15 +13,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
-
-from __future__ import print_function
 
 import unittest
 from datetime import datetime, timedelta
@@ -33,7 +25,7 @@ from rucio.client.replicaclient import ReplicaClient
 from rucio.common.config import config_get_bool
 from rucio.common.types import InternalScope
 from rucio.core.replica import (update_replica_state, list_replicas, list_bad_replicas_status)
-from rucio.core.rse import get_rse_id, add_rse_attribute
+from rucio.core.rse import add_rse_attribute
 from rucio.core.did import set_metadata
 from rucio.daemons.replicarecoverer.suspicious_replica_recoverer import run, stop
 from rucio.db.sqla.constants import DIDType, BadFilesStatus, ReplicaState
@@ -41,8 +33,7 @@ from rucio.tests.common import execute, file_generator
 from rucio.tests.common_server import get_vo
 
 
-@pytest.mark.dirty
-@pytest.mark.noparallel(reason='uses pre-defined RSE')
+@pytest.mark.usefixtures("rse_factory_unittest")
 class TestReplicaRecoverer(unittest.TestCase):
 
     def setUp(self):
@@ -52,16 +43,16 @@ class TestReplicaRecoverer(unittest.TestCase):
             self.vo = {}
 
         self.replica_client = ReplicaClient()
+        assert hasattr(self, "rse_factory")
+        rse_factory = self.rse_factory
 
         # Using two test RSEs
-        self.rse4suspicious = 'MOCK_SUSPICIOUS'
-        self.rse4suspicious_id = get_rse_id(self.rse4suspicious, **self.vo)
-        self.rse4recovery = 'MOCK_RECOVERY'
-        self.rse4recovery_id = get_rse_id(self.rse4recovery, **self.vo)
+        self.rse4suspicious, self.rse4suspicious_id = rse_factory.make_posix_rse(deterministic=True, **self.vo)
+        self.rse4recovery, self.rse4recovery_id = rse_factory.make_posix_rse(deterministic=True, **self.vo)
         self.scope = 'mock'
         self.internal_scope = InternalScope(self.scope, **self.vo)
 
-        # For testing, we create 3 files and upload them to Rucio to two test RSEs.
+        # For testing, we create 5 files and upload them to Rucio to two test RSEs.
         self.tmp_file1 = file_generator()
         self.tmp_file2 = file_generator()
         self.tmp_file3 = file_generator()
@@ -75,6 +66,7 @@ class TestReplicaRecoverer(unittest.TestCase):
             cmd = 'rucio -v upload --rse {0} --scope {1} {2} {3} {4} {5} {6}'.format(rse, self.scope, self.tmp_file1, self.tmp_file2, self.tmp_file3, self.tmp_file4, self.tmp_file5)
             exitcode, out, err = execute(cmd)
 
+            print(exitcode, out, err)
             # checking if Rucio upload went OK
             assert exitcode == 0
 
@@ -98,24 +90,26 @@ class TestReplicaRecoverer(unittest.TestCase):
 
         # Changing the replica statuses as follows:
         # ----------------------------------------------------------------------------------------------------------------------------------
-        # Name         State(s) declared on MOCK_RECOVERY       State(s) declared on MOCK_SUSPICIOUS        Metadata "datatype"
+        # Name         State(s) declared on rse4recovery       State(s) declared on rse4suspicious        Metadata "datatype"
         # ----------------------------------------------------------------------------------------------------------------------------------
         # tmp_file1    available                                suspicious (available)
         # tmp_file2    available                                suspicious + bad (unavailable)
         # tmp_file3    unavailable                              suspicious (available)                      RAW
-        # tmp_file4    unavailable                              suspicious (available)                      testtypedeclarebad
+        # tmp_file4    unavailable                              suspicious (unavailable)                    testtypedeclarebad
         # tmp_file5    unavailable                              suspicious (available)                      testtypenopolicy
         # ----------------------------------------------------------------------------------------------------------------------------------
 
         for replica in replicalist:
             suspicious_pfns = replica['rses'][self.rse4suspicious_id]
+            #  Declare each file as suspicious multiple times, except for tmp_file6
             for i in range(3):
                 print("Declaring suspicious file replica: " + suspicious_pfns[0])
-                self.replica_client.declare_suspicious_file_replicas([suspicious_pfns[0], ], 'This is a good reason.')
+                # The reason must contain the word "checksum", so that the replica can be declared bad.
+                self.replica_client.declare_suspicious_file_replicas([suspicious_pfns[0], ], 'checksum')
                 sleep(1)
             if replica['name'] == path.basename(self.tmp_file2):
                 print("Declaring bad file replica: " + suspicious_pfns[0])
-                self.replica_client.declare_bad_file_replicas([suspicious_pfns[0], ], 'This is a good reason')
+                self.replica_client.declare_bad_file_replicas([suspicious_pfns[0], ], 'checksum')
             if replica['name'] == path.basename(self.tmp_file3):
                 print("Updating replica state as unavailable: " + replica['rses'][self.rse4recovery_id][0])
                 update_replica_state(self.rse4recovery_id, self.internal_scope, path.basename(self.tmp_file3), ReplicaState.UNAVAILABLE)
@@ -147,7 +141,7 @@ class TestReplicaRecoverer(unittest.TestCase):
                 assert replica['states'][self.rse4suspicious_id] == 'AVAILABLE'
                 assert (self.rse4recovery_id in replica['states']) is False
 
-        # Checking if only self.tmp_file2 is declared as 'BAD'
+        # Checking if self.tmp_file2 and self.tmp_file6 were declared as 'BAD'
         self.from_date = datetime.now() - timedelta(days=1)
         bad_replicas_list = list_bad_replicas_status(rse_id=self.rse4suspicious_id, younger_than=self.from_date, **self.vo)
         bad_checklist = [(badf['name'], badf['rse_id'], badf['state']) for badf in bad_replicas_list]
@@ -167,7 +161,7 @@ class TestReplicaRecoverer(unittest.TestCase):
         assert (path.basename(self.tmp_file4), self.rse4recovery_id, BadFilesStatus.BAD) not in bad_checklist
         assert (path.basename(self.tmp_file5), self.rse4recovery_id, BadFilesStatus.BAD) not in bad_checklist
 
-        # On purpose not checking for status to be declared 'SUSPICIOUS' on MOCK_SUSPICIOUS.
+        # On purpose not checking for status to be declared 'SUSPICIOUS' on rse4suspicious.
         # The only existing function (to date) gathering info about 'SUSPICIOUS' replicas
         # is used (currently only) from the tested replica_recoverer itself.
 
@@ -177,20 +171,20 @@ class TestReplicaRecoverer(unittest.TestCase):
             setUp function (above) is supposed to run first
             (nose does this automatically):
 
-            - uploads 6 test files to two test RSEs ('MOCK_RECOVERY', 'MOCK_SUSPICIOUS')
+            - uploads 6 test files to two test RSEs (rse4recovery, rse4suspicious)
             - prepares their statuses to be as follows:
 
             # ----------------------------------------------------------------------------------------------------------------------------------
-            # Name         State(s) declared on MOCK_RECOVERY       State(s) declared on MOCK_SUSPICIOUS        Metadata "datatype"
+            # Name         State(s) declared on rse4recovery       State(s) declared on rse4suspicious        Metadata "datatype"
             # ----------------------------------------------------------------------------------------------------------------------------------
-            # tmp_file1    available                                suspicious (available)
+            # tmp_file1    available                                suspicious (unavailable)
             # tmp_file2    available                                suspicious + bad (unavailable)
             # tmp_file3    unavailable                              suspicious (available)                      RAW
-            # tmp_file4    unavailable                              suspicious (available)                      testtypedeclare_bad
+            # tmp_file4    unavailable                              suspicious (available)                      testtypedeclarebad
             # tmp_file5    unavailable                              suspicious (available)                      testtypenopolicy
             # ----------------------------------------------------------------------------------------------------------------------------------
 
-            - Explaination: Suspicious replicas that are the last remaining copy (unavailable on MOCK_RECOVERY) are handeled differently depending
+            - Explaination: Suspicious replicas that are the last remaining copy (unavailable on rse4recovery) are handeled differently depending
                             by their metadata "datatype". RAW files have the poilcy to be ignored. testtype_declare_bad files are of a fictional
                             type that has the policy of being declared bad. testtype_nopolicy files are of a fictional type that doesn't have a
                             policy specified, meaning they should be ignored by default.
@@ -201,26 +195,25 @@ class TestReplicaRecoverer(unittest.TestCase):
 
             Concluding:
 
-            - checks that tmp_file1 and tmp_file4 were declared as 'BAD' on 'MOCK_SUSPICIOUS'
+            - checks that tmp_file1, tmp_file4 and tmp_file6 were declared as 'BAD' on rse4suspicious
 
         """
 
-        # Run replica recoverer once
         try:
-            run(once=True, younger_than=1, nattempts=2, limit_suspicious_files_on_rse=5)
+            run(once=True, younger_than=1, nattempts=2, limit_suspicious_files_on_rse=5, sleep_time=0, active_mode=True)
         except KeyboardInterrupt:
             stop()
 
         # Checking the outcome:
-        # we expect to see only one change, i.e. tmp_file1 declared as bad on MOCK_SUSPICIOUS
+        # We expect to see three changes: tmp_file1 and tmp_file4 should be declared as bad on rse4suspicious
         # ----------------------------------------------------------------------------------------------------------------------------------
-        # Name         State(s) declared on MOCK_RECOVERY       State(s) declared on MOCK_SUSPICIOUS        Metadata "datatype"
+        # Name         State(s) declared on rse4recovery       State(s) declared on rse4suspicious        Metadata "datatype"
         # ----------------------------------------------------------------------------------------------------------------------------------
         # tmp_file1    available                                suspicious + bad (unavailable)
         # tmp_file2    available                                suspicious + bad (unavailable)
         # tmp_file3    unavailable                              suspicious (available)                      RAW
-        # tmp_file4    unavailable                              suspicious + bad (unvailable)               test_type_declare_bad
-        # tmp_file5    unavailable                              suspicious (available)                      test_type_ignore
+        # tmp_file4    unavailable                              suspicious + bad (unavailable)              testtypedeclarebad
+        # tmp_file5    unavailable                              suspicious (available)                      testtypenopolicy
         # ----------------------------------------------------------------------------------------------------------------------------------
 
         # Gather replica info after replica_recoverer has run.
@@ -240,7 +233,7 @@ class TestReplicaRecoverer(unittest.TestCase):
                 assert replica['states'][self.rse4suspicious_id] == 'AVAILABLE'
                 assert (self.rse4recovery_id in replica['states']) is False
 
-        # Checking if replicas declared as 'BAD'
+        # Checking if replicas were declared as 'BAD'
         bad_replicas_list = list_bad_replicas_status(rse_id=self.rse4suspicious_id, younger_than=self.from_date, **self.vo)
         bad_checklist = [(badf['name'], badf['rse_id'], badf['state']) for badf in bad_replicas_list]
 
