@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,29 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2018
-# - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2015
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2013
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2021
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2013
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2015
-# - Wen Guan <wen.guan@cern.ch>, 2014-2015
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
-# - Frank Berghaus <frank.berghaus@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2019
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
-# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
-# - Mayank Sharma <mayank.sharma@cern.ch>, 2022
-
-from __future__ import print_function
 
 import unittest
 
@@ -46,13 +23,15 @@ from rucio.common import exception
 from rucio.common.config import config_get_bool
 from rucio.common.exception import (Duplicate, RSENotFound, RSEProtocolNotSupported,
                                     InvalidObject, ResourceTemporaryUnavailable,
-                                    RSEAttributeNotFound, RSEOperationNotSupported)
+                                    RSEAttributeNotFound, RSEOperationNotSupported,
+                                    InputValidationError)
 from rucio.common.utils import generate_uuid
 from rucio.core.rse import (add_rse, get_rse_id, del_rse, restore_rse, list_rses,
                             rse_exists, add_rse_attribute, list_rse_attributes,
                             set_rse_transfer_limits, get_rse_transfer_limits,
                             delete_rse_transfer_limits, get_rse_protocols,
-                            del_rse_attribute, get_rse_attribute, get_rse, rse_is_empty)
+                            del_rse_attribute, get_rse_attribute, get_rse, rse_is_empty,
+                            update_rse)
 from rucio.db.sqla import session, models
 from rucio.db.sqla.constants import RSEType
 from rucio.rse import rsemanager as mgr
@@ -458,6 +437,13 @@ class TestRSEClient(unittest.TestCase):
         with pytest.raises(ResourceTemporaryUnavailable):
             replica_client.add_replicas(rse=renamed_rse, files=files2, ignore_availability=False)
 
+    def test_update_rse_availability_all_false(self):
+        """ RSE (CLIENTS): update rse should be able to set all availability options to False."""
+        rse = rse_name_generator()
+        ret = self.client.add_rse(rse)
+        assert ret
+        self.client.update_rse(rse, {"availability_read": False, "availability_write": False, "availability_delete": False})
+
     def test_list_rses(self):
         """ RSE (CLIENTS): try to list rses."""
         rse_list = [rse_name_generator() for i in range(5)]
@@ -1485,3 +1471,16 @@ class TestRSEClient(unittest.TestCase):
         rse_name = rse_name_generator()
         with pytest.raises(RSENotFound):
             self.client.delete_rse(rse=rse_name)
+
+    def test_rse_update_unsupported_option(self):
+        """ RSE(CLIENT): update with an unsupported option should throw an exception"""
+        rse_name = rse_name_generator()
+        add_rse(rse_name, **self.vo)
+        rse_id = get_rse_id(rse_name, **self.vo)
+
+        update_rse(rse_id, parameters={'city': 'Berlin'})
+        assert get_rse(rse_id)['city'] == 'Berlin'
+
+        with pytest.raises(InputValidationError):
+            update_rse(rse_id, parameters={'city': 'Not Berlin', 'non_existing_option': 3})
+        assert get_rse(rse_id)['city'] == 'Berlin'
