@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,16 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Martin Barisits <martin.barisits@cern.ch>, 2013-2021
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2014
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Patrick Austin <patrick.austin@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
 
 import unittest
 from random import choice
@@ -55,10 +45,10 @@ class TestRSEExpressionParserCore(unittest.TestCase):
     def setUp(self):
         if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
             self.vo = {'vo': get_vo()}
-            self.filter = {'filter': self.vo}
+            self.filter = {'filter_': self.vo}
         else:
             self.vo = {}
-            self.filter = {'filter': {'vo': 'def'}}
+            self.filter = {'filter_': {'vo': 'def'}}
 
         self.rse1 = rse_name_generator()
         self.rse2 = rse_name_generator()
@@ -126,7 +116,7 @@ class TestRSEExpressionParserCore(unittest.TestCase):
 
     def test_all_rse(self):
         """ RSE_EXPRESSION_PARSER (CORE) Test reference on all RSE """
-        all_rses = rse.list_rses(filters=self.filter['filter'])
+        all_rses = rse.list_rses(filters=self.filter['filter_'])
         value = sorted(rse_expression_parser.parse_expression("*", **self.filter), key=lambda rse: rse['rse'])
         expected = sorted(all_rses, key=lambda rse: rse['rse'])
         assert value == expected
