@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,12 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Wen Guan <wen.guan@cern.ch>, 2014
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2014-2015
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 import os
 import shutil
@@ -38,6 +32,7 @@ from rucio.tests.rsemgr_api_test import MgrTestCases
 class TestRseGFAL2(unittest.TestCase):
     tmpdir = None
     user = None
+    impl = 'gfal'
 
     @classmethod
     def setUpClass(cls):
@@ -132,9 +127,9 @@ class TestRseGFAL2(unittest.TestCase):
         """GFAL2 (RSE/PROTOCOLS): Creating Mgr-instance """
         self.tmpdir = TestRseGFAL2.tmpdir
         self.rse_id = 'FZK-LCG2_SCRATCHDISK'
-        self.mtc = MgrTestCases(self.tmpdir, 'FZK-LCG2_SCRATCHDISK', TestRseGFAL2.user, TestRseGFAL2.static_file)
+        self.mtc = MgrTestCases(self.tmpdir, 'FZK-LCG2_SCRATCHDISK', TestRseGFAL2.user, TestRseGFAL2.static_file, impl=TestRseGFAL2.impl)
         # self.mtc = MgrTestCases(self.tmpdir, 'FZK-LCG2_SCRATCHDISK', TestRseGFAL2.user, "srm://atlassrm-fzk.gridka.de/pnfs/gridka.de/atlas/disk-only/atlasscratchdisk/user/wguan/rucio.test.2")
-        self.mtc.setup_scheme('srm', 'gfal')
+        self.mtc.setup_scheme('srm')
 
     # Mgr-Tests: GET
     def test_multi_get_mgr_ok(self):
