@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,17 +12,11 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2014
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2017-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2017
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - Mayank Sharma <mayank.sharma@cern.ch>, 2021
 
 import datetime
 import time
 import uuid
+import logging
 import pytest
 import json
 from rucio.common.schema.generic import IPv4orIPv6
@@ -30,6 +24,9 @@ from rucio.core.trace import SCHEMAS, validate_schema
 from rucio.common.exception import InvalidObject
 
 
+LOGGER = logging.getLogger(__name__)
+
+
 def test_submit_trace(rest_client):
     """ TRACE (REST): submit a trace via POST """
     payload = {'uuid': str(uuid.uuid4()),  # str, because not JSON serializable
@@ -87,3 +84,14 @@ def test_trace_ip():
     for obj in invalid_obj:
         with pytest.raises(InvalidObject):
             validate_schema(obj)
+
+
+def test_non_existant_event_type_validation_rejection(caplog):
+    """
+    Test if an incoming trace with a non-supported schema logs a warning
+    instead of printing entire stack-trace
+    """
+    event_type = "put_new_type"
+    obj = json.dumps({"eventType": f"{event_type}"})
+    validate_schema(obj)
+    assert f"schema for eventType {event_type} is not currently supported" in caplog.text
