@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2013-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,44 +12,15 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2013-2021
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2013-2018
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2019
-# - Wen Guan <wen.guan@cern.ch>, 2014-2016
-# - Martin Barisits <martin.barisits@cern.ch>, 2015-2022
-# - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
-# - Eric Vaandering <ewv@fnal.gov>, 2018
-# - dciangot <diego.ciangottini@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
-# - Dilaksun Bavarajan <dilaksun.bavarajan@cern.ch>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Brandon White <bjwhite@fnal.gov>, 2019
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2020
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2020-2021
-# - Sahan Dilshan <32576163+sahandilshan@users.noreply.github.com>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021-2022
-# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
-# - Rob Barnsley <rob.barnsley@skao.int>, 2022
 
-from __future__ import absolute_import, division
 import datetime
 import json
-try:
-    from json.decoder import JSONDecodeError
-except ImportError:
-    JSONDecodeError = ValueError
 import functools
 import logging
-import time
 import traceback
-try:
-    from urlparse import urlparse  # py2
-except ImportError:
-    from urllib.parse import urlparse  # py3
 import uuid
+from typing import Any, Callable, Tuple, TYPE_CHECKING
+from urllib.parse import urlparse
 
 import requests
 from configparser import NoOptionError, NoSectionError
@@ -63,14 +34,17 @@ from rucio.common.cache import make_region_memcached
 from rucio.common.config import config_get, config_get_bool
 from rucio.common.constants import FTS_JOB_TYPE, FTS_STATE, FTS_COMPLETE_STATE
 from rucio.common.exception import TransferToolTimeout, TransferToolWrongAnswer, DuplicateFileTransferSubmission
-from rucio.common.utils import APIEncoder, chunks, set_checksum_value
+from rucio.common.utils import APIEncoder, chunks, PREFERRED_CHECKSUM
 from rucio.core.request import get_source_rse, get_transfer_error
 from rucio.core.rse import get_rse_supported_checksums_from_attributes
 from rucio.core.oidc import get_token_for_account_operation
-from rucio.core.monitor import record_counter, record_timer, MultiCounter
+from rucio.core.monitor import record_counter, Timer, MultiCounter
 from rucio.transfertool.transfertool import Transfertool, TransferToolBuilder, TransferStatusReport
 from rucio.db.sqla.constants import RequestState
 
+if TYPE_CHECKING:
+    from rucio.core.transfer import DirectTransferDefinition
+
 logging.getLogger("requests").setLevel(logging.CRITICAL)
 disable_warnings()
 
@@ -103,6 +77,27 @@ FINAL_FTS_JOB_STATES = (FTS_STATE.FAILED, FTS_STATE.CANCELED, FTS_STATE.FINISHED
 FINAL_FTS_FILE_STATES = (FTS_STATE.FAILED, FTS_STATE.CANCELED, FTS_STATE.FINISHED, FTS_STATE.NOT_USED)
 
 
+def _configured_source_strategy(activity: str, logger: Callable[..., Any]) -> str:
+    """
+    Retrieve from the configuration the source selection strategy for the given activity
+    """
+    try:
+        default_source_strategy = config_get(section='conveyor', option='default-source-strategy')
+    except (NoOptionError, NoSectionError, RuntimeError):
+        default_source_strategy = 'orderly'
+
+    try:
+        activity_source_strategy = config_get(section='conveyor', option='activity-source-strategy')
+        activity_source_strategy = loads(activity_source_strategy)
+    except (NoOptionError, NoSectionError, RuntimeError):
+        activity_source_strategy = {}
+    except ValueError:
+        logger(logging.WARNING, 'activity_source_strategy not properly defined')
+        activity_source_strategy = {}
+
+    return activity_source_strategy.get(str(activity), default_source_strategy)
+
+
 def oidc_supported(transfer_hop) -> bool:
     """
     checking OIDC AuthN/Z support per destination and source RSEs;
@@ -119,59 +114,52 @@ def oidc_supported(transfer_hop) -> bool:
     return True
 
 
-def checksum_validation_strategy(src_attributes, dst_attributes, logger):
+def _checksum_validation_strategy(
+    transfer: "DirectTransferDefinition",
+    logger: Callable[..., Any],
+) -> Tuple[str, str]:
     """
-    Compute the checksum validation strategy (none, source, destination or both) and the
-    supported checksums from the attributes of the source and destination RSE.
+    Compute the checksum validation strategy (none, source, destination or both) and the checksum
+    to use (in the format accepted by FTS) from the attributes of the source and destination RSE.
     """
-    source_supported_checksums = get_rse_supported_checksums_from_attributes(src_attributes)
-    dest_supported_checksums = get_rse_supported_checksums_from_attributes(dst_attributes)
-    common_checksum_names = set(source_supported_checksums).intersection(dest_supported_checksums)
-
-    verify_checksum = 'both'
-    if not dst_attributes.get('verify_checksum', True):
-        if not src_attributes.get('verify_checksum', True):
-            verify_checksum = 'none'
-        else:
-            verify_checksum = 'source'
+    src_attributes = transfer.src.rse.attributes
+    if src_attributes.get('verify_checksum', True):
+        src_checksums = set(get_rse_supported_checksums_from_attributes(src_attributes))
     else:
-        if not src_attributes.get('verify_checksum', True):
-            verify_checksum = 'destination'
-        else:
-            verify_checksum = 'both'
+        src_checksums = set()
 
-    if len(common_checksum_names) == 0:
+    dst_attributes = transfer.dst.rse.attributes
+    if dst_attributes.get('verify_checksum', True):
+        dst_checksums = set(get_rse_supported_checksums_from_attributes(dst_attributes))
+    else:
+        dst_checksums = set()
+
+    intersection = src_checksums.intersection(dst_checksums)
+
+    if intersection:
+        strategy, possible_checksums = 'both', intersection
+    elif dst_checksums:
+        # The prioritization of destination over source here is desired, not random
         logger(logging.INFO, 'No common checksum method. Verifying destination only.')
-        verify_checksum = 'destination'
-
-    if source_supported_checksums == ['none']:
-        if dest_supported_checksums == ['none']:
-            # both endpoints support none
-            verify_checksum = 'none'
-        else:
-            # src supports none but dst does
-            verify_checksum = 'destination'
+        strategy, possible_checksums = 'target', dst_checksums
+    elif src_checksums:
+        logger(logging.INFO, 'No common checksum method. Verifying source only.')
+        strategy, possible_checksums = 'source', src_checksums
     else:
-        if dest_supported_checksums == ['none']:
-            # source supports some but destination does not
-            verify_checksum = 'source'
-        else:
-            if len(common_checksum_names) == 0:
-                # source and dst support some bot none in common (dst priority)
-                verify_checksum = 'destination'
-            else:
-                # Don't override the value in the file_metadata
-                pass
+        logger(logging.INFO, 'No common checksum method. Not verifying source nor destination.')
+        strategy, possible_checksums = 'none', set()
 
-    checksums_to_use = ['none']
-    if verify_checksum == 'both':
-        checksums_to_use = common_checksum_names
-    elif verify_checksum == 'source':
-        checksums_to_use = source_supported_checksums
-    elif verify_checksum == 'destination':
-        checksums_to_use = dest_supported_checksums
+    checksum_to_use = None
+    for checksum_name in possible_checksums:
+        checksum_value = getattr(transfer.rws, checksum_name, '')
+        if not checksum_value:
+            continue
 
-    return verify_checksum, checksums_to_use
+        checksum_to_use = '%s:%s' % (checksum_name.upper(), checksum_value)
+        if checksum_name == PREFERRED_CHECKSUM:
+            break
+
+    return strategy, checksum_to_use
 
 
 def job_params_for_fts_transfer(transfer, bring_online, default_lifetime, archive_timeout_override, max_time_in_queue, logger, multihop=False):
@@ -196,8 +184,7 @@ def job_params_for_fts_transfer(transfer, bring_online, default_lifetime, archiv
     strict_copy = transfer.dst.rse.attributes.get('strict_copy', False)
     archive_timeout = transfer.dst.rse.attributes.get('archive_timeout', None)
 
-    verify_checksum, checksums_to_use = checksum_validation_strategy(transfer.src.rse.attributes, transfer.dst.rse.attributes, logger=logger)
-    transfer['checksums_to_use'] = checksums_to_use
+    verify_checksum, _checksum_to_use = _checksum_validation_strategy(transfer, logger=logger)
 
     job_params = {'account': transfer.rws.account,
                   'verify_checksum': verify_checksum,
@@ -262,24 +249,6 @@ def bulk_group_transfers(transfer_paths, policy='rule', group_bulk=200, source_s
     grouped_transfers = {}
     grouped_jobs = []
 
-    try:
-        default_source_strategy = config_get(section='conveyor', option='default-source-strategy')
-    except (NoOptionError, NoSectionError, RuntimeError):
-        default_source_strategy = 'orderly'
-
-    try:
-        activity_source_strategy = config_get(section='conveyor', option='activity-source-strategy')
-        activity_source_strategy = loads(activity_source_strategy)
-    except (NoOptionError, NoSectionError, RuntimeError):
-        activity_source_strategy = {}
-    except ValueError:
-        logger(logging.WARNING, 'activity_source_strategy not properly defined')
-        activity_source_strategy = {}
-
-    for transfer_path in transfer_paths:
-        for i, transfer in enumerate(transfer_path):
-            transfer['selection_strategy'] = source_strategy if source_strategy else activity_source_strategy.get(str(transfer.rws.activity), default_source_strategy)
-
     _build_job_params = functools.partial(job_params_for_fts_transfer,
                                           bring_online=bring_online,
                                           default_lifetime=default_lifetime,
@@ -291,14 +260,19 @@ def bulk_group_transfers(transfer_paths, policy='rule', group_bulk=200, source_s
             # for multihop transfers, all the path is submitted as a separate job
             job_params = _build_job_params(transfer_path[-1], multihop=True)
 
+            overwrite_hop = True
             for transfer in transfer_path[:-1]:
                 hop_params = _build_job_params(transfer, multihop=True)
                 # Only allow overwrite if all transfers in multihop allow it
                 job_params['overwrite'] = hop_params['overwrite'] and job_params['overwrite']
+                # Allow overwrite_hop if all intermediate hops allow it (ignoring the last hop)
+                overwrite_hop = hop_params['overwrite'] and overwrite_hop
                 # Activate bring_online if it was requested by first hop (it is a multihop starting at a tape)
                 # We don't allow multihop via a tape, so bring_online should not be set on any other hop
                 if transfer is transfer_path[0] and hop_params['bring_online']:
                     job_params['bring_online'] = hop_params['bring_online']
+            if not job_params['overwrite'] and overwrite_hop:
+                job_params['overwrite_hop'] = overwrite_hop
 
             group_key = 'multihop_%s' % transfer_path[-1].rws.request_id
             grouped_transfers[group_key] = {'transfers': transfer_path[0:group_bulk], 'job_params': job_params}
@@ -379,6 +353,7 @@ class Fts3TransferStatusReport(TransferStatusReport):
         # Initialized in child class initialize():
         self._reason = None
         self._src_rse = None
+        self._fts_address = self.external_host
         # Supported db fields bellow:
         self.state = None
         self.external_id = None
@@ -414,7 +389,7 @@ class Fts3TransferStatusReport(TransferStatusReport):
         return fields
 
     def _transfer_link(self):
-        return '%s/fts3/ftsmon/#/job/%s' % (self.external_host.replace('8446', '8449'), self._transfer_id)
+        return '%s/fts3/ftsmon/#/job/%s' % (self._fts_address.replace('8446', '8449'), self._transfer_id)
 
     def _find_attribute_updates(self, request, new_state, reason, overwrite_corrupted_files):
         attributes = None
@@ -530,11 +505,15 @@ class FTS3CompletionMessageTransferStatusReport(Fts3TransferStatusReport):
         transfer_id = self._transfer_id
         if new_state:
             request = self.request(session)
-            if request['external_id'] == transfer_id and request['state'] != new_state:
+            if not request:
+                logger(logging.WARNING, '%s: no request with this id in the database. Skipping. external_id: %s (%s). new_state: %s', request_id, transfer_id, self.external_host, new_state)
+                return
+            if request and request['external_id'] == transfer_id and request['state'] != new_state:
                 src_rse_name, src_rse_id = self._find_used_source_rse(session, logger)
 
                 self._reason = reason
                 self._src_rse = src_rse_name
+                self._fts_address = request['external_host'] or self._fts_address
 
                 self.state = new_state
                 self.external_id = transfer_id
@@ -544,6 +523,8 @@ class FTS3CompletionMessageTransferStatusReport(Fts3TransferStatusReport):
                 self.staging_finished_at = None
                 self.source_rse_id = src_rse_id
                 self.err_msg = get_transfer_error(self.state, reason)
+                if self.err_msg and self._file_metadata.get('src_type') == "TAPE":
+                    self.err_msg = '[TAPE SOURCE] ' + self.err_msg
                 self.attributes = self._find_attribute_updates(
                     request=request,
                     new_state=new_state,
@@ -592,10 +573,12 @@ class FTS3ApiTransferStatusReport(Fts3TransferStatusReport):
         if file_state_is_final:
             if file_state == FTS_STATE.FINISHED:
                 new_state = RequestState.DONE
-            elif job_state_is_final and file_state == FTS_STATE.FAILED \
-                    and self._is_recoverable_fts_overwrite_error(self.request(session), reason, self._file_metadata):
-                new_state = RequestState.DONE
-            elif job_state_is_final and file_state in (FTS_STATE.FAILED, FTS_STATE.CANCELED):
+            elif file_state == FTS_STATE.FAILED and not self._multi_sources:  # for multi-source transfers we must wait for the job to be in a final state
+                if self._is_recoverable_fts_overwrite_error(self.request(session), reason, self._file_metadata):
+                    new_state = RequestState.DONE
+                else:
+                    new_state = RequestState.FAILED
+            elif job_state_is_final and file_state == FTS_STATE.CANCELED:
                 new_state = RequestState.FAILED
             elif job_state_is_final and file_state == FTS_STATE.NOT_USED:
                 if job_state == FTS_STATE.FINISHED:
@@ -610,6 +593,9 @@ class FTS3ApiTransferStatusReport(Fts3TransferStatusReport):
         transfer_id = self._transfer_id
         if new_state:
             request = self.request(session)
+            if not request:
+                logger(logging.WARNING, '%s: no request with this id in the database. Skipping. external_id: %s (%s). new_state: %s', request_id, transfer_id, self.external_host, new_state)
+                return
             if request['external_id'] == transfer_id and request['state'] != new_state:
                 src_rse_name, src_rse_id = self._find_used_source_rse(session, logger)
 
@@ -624,6 +610,8 @@ class FTS3ApiTransferStatusReport(Fts3TransferStatusReport):
                 self.staging_finished_at = datetime.datetime.strptime(file_response['staging_finished'], '%Y-%m-%dT%H:%M:%S') if file_response['staging_finished'] else None
                 self.source_rse_id = src_rse_id
                 self.err_msg = get_transfer_error(self.state, reason)
+                if self.err_msg and self._file_metadata.get('src_type') == "TAPE":
+                    self.err_msg = '[TAPE SOURCE] ' + self.err_msg
                 self.attributes = self._find_attribute_updates(
                     request=request,
                     new_state=new_state,
@@ -641,6 +629,8 @@ class FTS3Transfertool(Transfertool):
     FTS3 implementation of a Rucio transfertool
     """
 
+    external_name = 'fts3'
+
     def __init__(self, external_host, oidc_account=None, vo=None, group_bulk=1, group_policy='rule', source_strategy=None,
                  max_time_in_queue=None, bring_online=43200, default_lifetime=172800, archive_timeout_override=None,
                  logger=logging.log):
@@ -699,28 +689,32 @@ class FTS3Transfertool(Transfertool):
         if config_get_bool('common', 'multi_vo', False, None):
             vo = transfer_path[-1].rws.scope.vo
 
-        all_hops_have_an_fts_attr = True
+        sub_path = []
         fts_hosts = []
         for hop in transfer_path:
-            fts_hosts = hop.dst.rse.attributes.get('fts', None)
+            src_and_dst_have_fts_attribute = hop.src.rse.attributes.get('fts', None) is not None and hop.dst.rse.attributes.get('fts', None) is not None
+            hosts = hop.dst.rse.attributes.get('fts', None)
             if hop.src.rse.attributes.get('sign_url', None) == 'gcs':
-                fts_hosts = hop.src.rse.attributes.get('fts', None)
-            fts_hosts = fts_hosts.split(",") if fts_hosts else []
+                hosts = hop.src.rse.attributes.get('fts', None)
+            hosts = hosts.split(",") if hosts else []
 
-            if not fts_hosts:
-                all_hops_have_an_fts_attr = False
+            if src_and_dst_have_fts_attribute and hosts:
+                fts_hosts = hosts
+                sub_path.append(hop)
+            else:
                 break
 
-        if not fts_hosts or not all_hops_have_an_fts_attr:
-            logger(logging.WARN, 'FTS3Transfertool cannot be used to submit transfer {}: some hops do not have an fts attribute'.format([str(hop) for hop in transfer_path]))
-            return None
+        if len(sub_path) < len(transfer_path):
+            logger(logging.INFO, 'FTS3Transfertool can only submit {} hops from {}'.format(len(sub_path), [str(hop) for hop in transfer_path]))
 
-        oidc_account = None
-        if all(oidc_supported(t) for t in transfer_path):
-            logger(logging.DEBUG, 'OAuth2/OIDC available for transfer {}'.format([str(hop) for hop in transfer_path]))
-            oidc_account = transfer_path[-1].rws.account
-
-        return TransferToolBuilder(cls, external_host=fts_hosts[0], oidc_account=oidc_account, vo=vo)
+        if sub_path:
+            oidc_account = None
+            if all(oidc_supported(t) for t in sub_path):
+                logger(logging.DEBUG, 'OAuth2/OIDC available for transfer {}'.format([str(hop) for hop in sub_path]))
+                oidc_account = transfer_path[-1].rws.account
+            return sub_path, TransferToolBuilder(cls, external_host=fts_hosts[0], oidc_account=oidc_account, vo=vo)
+        else:
+            return [], None
 
     def group_into_submit_jobs(self, transfer_paths):
         jobs = bulk_group_transfers(
@@ -735,9 +729,9 @@ class FTS3Transfertool(Transfertool):
         )
         return jobs
 
-    @classmethod
-    def __file_from_transfer(cls, transfer, job_params):
+    def __file_from_transfer(self, transfer):
         rws = transfer.rws
+        _verify_checksum, checksum_to_use = _checksum_validation_strategy(transfer, logger=self.logger)
         t_file = {
             'sources': [s[1] for s in transfer.legacy_sources],
             'destinations': [transfer.dest_url],
@@ -758,14 +752,10 @@ class FTS3Transfertool(Transfertool):
                 'adler32': rws.adler32
             },
             'filesize': rws.byte_count,
-            'checksum': None,
-            'verify_checksum': job_params['verify_checksum'],
-            'selection_strategy': transfer['selection_strategy'],
-            'request_type': rws.request_type,
+            'checksum': checksum_to_use,
+            'selection_strategy': self.source_strategy if self.source_strategy else _configured_source_strategy(transfer.rws.activity, logger=self.logger),
             'activity': rws.activity
         }
-        if t_file['verify_checksum'] != 'none':
-            set_checksum_value(t_file, transfer['checksums_to_use'])
         return t_file
 
     def submit(self, transfers, job_params, timeout=None):
@@ -777,7 +767,6 @@ class FTS3Transfertool(Transfertool):
         :param timeout:      Timeout in seconds.
         :returns:            FTS transfer identifier.
         """
-        start_time = time.time()
         files = []
         for transfer in transfers:
             if isinstance(transfer, dict):
@@ -786,7 +775,7 @@ class FTS3Transfertool(Transfertool):
                 files.append(transfer)
                 continue
 
-            files.append(self.__file_from_transfer(transfer, job_params))
+            files.append(self.__file_from_transfer(transfer))
 
         # FTS3 expects 'davs' as the scheme identifier instead of https
         for transfer_file in files:
@@ -824,7 +813,7 @@ class FTS3Transfertool(Transfertool):
 
         post_result = None
         try:
-            start_time = time.time()
+            timer = Timer()
             post_result = requests.post('%s/jobs' % self.external_host,
                                         verify=self.verify,
                                         cert=self.cert,
@@ -832,10 +821,10 @@ class FTS3Transfertool(Transfertool):
                                         headers=self.headers,
                                         timeout=timeout)
             labels = {'host': self.__extract_host(self.external_host)}
-            record_timer('transfertool.fts3.submit_transfer.{host}', (time.time() - start_time) * 1000 / len(files), labels=labels)
+            timer.record('transfertool.fts3.submit_transfer.{host}', divisor=len(files), labels=labels)
         except ReadTimeout as error:
             raise TransferToolTimeout(error)
-        except JSONDecodeError as error:
+        except json.JSONDecodeError as error:
             raise TransferToolWrongAnswer(error)
         except Exception as error:
             self.logger(logging.WARNING, 'Could not submit transfer to %s - %s' % (self.external_host, str(error)))
@@ -856,7 +845,7 @@ class FTS3Transfertool(Transfertool):
 
         if not transfer_id:
             raise TransferToolWrongAnswer('No transfer id returned by %s' % self.external_host)
-        record_timer('core.request.submit_transfers_fts3', (time.time() - start_time) * 1000 / len(transfers))
+        timer.record('core.request.submit_transfers_fts3', divisor=len(transfers))
         return transfer_id
 
     def cancel(self, transfer_ids, timeout=None):
@@ -1018,7 +1007,7 @@ class FTS3Transfertool(Transfertool):
                 responses = self.__bulk_query_responses(jobs_response, requests_by_eid)
             except ReadTimeout as error:
                 raise TransferToolTimeout(error)
-            except JSONDecodeError as error:
+            except json.JSONDecodeError as error:
                 raise TransferToolWrongAnswer(error)
             except Exception as error:
                 raise Exception("Failed to parse the job response: %s, error: %s" % (str(jobs), str(error)))
@@ -1209,7 +1198,7 @@ class FTS3Transfertool(Transfertool):
                                               timeout=5)
                 except ReadTimeout as error:
                     raise TransferToolTimeout(error)
-                except JSONDecodeError as error:
+                except json.JSONDecodeError as error:
                     raise TransferToolWrongAnswer(error)
                 except Exception as error:
                     self.logger(logging.WARNING, 'Could not get baseid and voname from %s - %s' % (self.external_host, str(error)))
