@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,18 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2018-2021
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Rahul Chauhan <omrahulchauhan@gmail.com>, 2021
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
-
-from __future__ import print_function
 
 import base64
 import logging
@@ -32,7 +20,7 @@ from re import search
 from typing import TYPE_CHECKING
 
 from flask import Flask, Blueprint, request, Response, redirect, render_template
-from six.moves.urllib.parse import urlparse
+from urllib.parse import urlparse
 from werkzeug.datastructures import Headers
 
 from rucio.api.authentication import get_auth_token_user_pass, get_auth_token_gss, get_auth_token_x509, \
@@ -420,6 +408,8 @@ class TokenOIDC(ErrorHandlingMethodView):
             response.headers.extend(headers)
             response.set_cookie('x-rucio-auth-token', value=result['token']['token'], domain=domain, path='/')
             response.set_cookie('rucio-auth-token-created-at', value=str(time.time()), domain=domain, path='/')
+            # response.set_cookie('x-rucio-auth-token', value=result['token']['token'])
+            # response.set_cookie('rucio-auth-token-created-at', value=str(time.time()))
             return response
         else:
             return '', 400, headers
