@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2014-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,17 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2018-2021
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2020
-# - Martin Barisits <martin.barisits@cern.ch>, 2020
-# - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
 
 import itertools
 import json
@@ -33,7 +22,6 @@ from time import time
 from typing import TYPE_CHECKING
 
 import flask
-import six
 from flask.views import MethodView
 from werkzeug.datastructures import Headers
 from werkzeug.exceptions import HTTPException
@@ -193,11 +181,7 @@ def error_headers(exc_cls: str, exc_msg):
     def strip_newlines(msg):
         if msg is None:
             return None
-        elif isinstance(msg, six.binary_type):
-            msg = six.ensure_text(msg, errors='replace')
-        elif not isinstance(msg, six.string_types):
-            # any objects will be converted to their string representation in unicode
-            msg = six.ensure_text(str(msg), errors='replace')
+
         return msg.replace('\n', ' ').replace('\r', ' ')
 
     exc_msg = strip_newlines(exc_msg)
