@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
-# Copyright 2020-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,12 +13,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2021
-# - Martin Barisits <martin.barisits@cern.ch>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2021
 
 import importlib
 
@@ -34,6 +28,7 @@ DEFAULT_ENDPOINTS = [
     'config',
     'credentials',
     'dids',
+    'dirac',
     'export',
     'heartbeats',
     'identities',
