@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,13 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2017
-# - Thomas Beermann <thomas.beermann@cern.ch>, 2012-2018
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2014-2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
 from typing import TYPE_CHECKING
 
@@ -51,31 +44,24 @@ class Ping(ErrorHandlingMethodView):
     @check_accept_header_wrapper_flask(['application/json'])
     def get(self):
         """
-        Ping the server and retrieve the server version.
-
-        .. :quickref: Ping; Ping the server.
-
-        **Example request**:
-
-        .. sourcecode:: http
-
-            GET /ping HTTP/1.1
-            Host: rucio-server.com
-            Accept: application/json
-
-        **Example response**:
-
-        .. sourcecode:: http
-
-            HTTP/1.1 200 OK
-            Vary: Accept
-            Content-Type: application/json
-
-            {"version": "1.15.0"}
-
-        :status 200: OK.
-        :status 406: Not Acceptable.
-        :returns: JSON dictionary with the version.
+        ---
+        summary: Ping
+        description: Ping the server and get data about it.
+        tags:
+          - Ping
+        responses:
+          200:
+            description: OK
+            content:
+              application/json:
+                schema:
+                  type: object
+                  properties:
+                    version:
+                      descripption: The server version.
+                      type: string
+          406:
+            description: Not acceptable
         """
         headers = self.get_headers()
         response = jsonify(version=version.version_string())
@@ -83,11 +69,11 @@ class Ping(ErrorHandlingMethodView):
         return response
 
 
-def blueprint(no_doc=True):
+def blueprint(with_doc=False):
     bp = Blueprint('ping', __name__, url_prefix='/ping')
 
     ping_view = Ping.as_view('ping')
-    if no_doc:
+    if not with_doc:
         # rule without trailing slash needs to be added before rule with trailing slash
         bp.add_url_rule('', view_func=ping_view, methods=['get', ])
     bp.add_url_rule('/', view_func=ping_view, methods=['get', ])
@@ -99,5 +85,5 @@ def blueprint(no_doc=True):
 def make_doc():
     """ Only used for sphinx documentation to add the prefix """
     doc_app = Flask(__name__)
-    doc_app.register_blueprint(blueprint(no_doc=False))
+    doc_app.register_blueprint(blueprint(with_doc=True))
     return doc_app
